@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2005-2016, PyInstaller Development Team.
+# Copyright (c) 2005-2018, PyInstaller Development Team.
 #
 # Distributed under the terms of the GNU General Public License with exception
 # for distributing bootloader.
@@ -21,9 +21,12 @@ import platform
 import shutil
 import sys
 
+import struct
+
 from PyInstaller.config import CONF
-from .. import is_darwin, is_win, compat
-from ..compat import EXTENSION_SUFFIXES, FileNotFoundError
+from .. import compat
+from ..compat import is_darwin, is_win, EXTENSION_SUFFIXES, \
+    open_file, is_py3, is_py37
 from ..depend import dylib
 from ..depend.bindepend import match_binding_redirect
 from ..utils import misc
@@ -94,12 +97,27 @@ def add_suffix_to_extensions(toc):
     new_toc = TOC()
     for inm, fnm, typ in toc:
         if typ == 'EXTENSION':
-            # In some rare cases extension might already contain suffix.
+            if is_py3:
+                # Change the dotted name into a relative path. This places C
+                # extensions in the Python-standard location. This only works
+                # in Python 3; see comments above
+                # ``sys.meta_path.append(CExtensionImporter())`` in
+                # ``pyimod03_importers``.
+                inm = inm.replace('.', os.sep)
+            # In some rare cases extension might already contain a suffix.
             # Skip it in this case.
-            if not inm.endswith(EXTENSION_SUFFIXES[0]):
-                # Use first suffix from the Python list of suffixes
-                # for C extensions.
-                inm = inm + EXTENSION_SUFFIXES[0]
+            if os.path.splitext(inm)[1] not in EXTENSION_SUFFIXES:
+                # Determine the base name of the file.
+                if is_py3:
+                    base_name = os.path.basename(inm)
+                else:
+                    base_name = inm.rsplit('.')[-1]
+                assert '.' not in base_name
+                # Use this file's existing extension. For extensions such as
+                # ``libzmq.cp36-win_amd64.pyd``, we can't use
+                # ``os.path.splitext``, which would give only the ```.pyd`` part
+                # of the extension.
+                inm = inm + os.path.basename(fnm)[len(base_name):]
 
         elif typ == 'DEPENDENCY':
             # Use the suffix from the filename.
@@ -122,12 +140,15 @@ def applyRedirects(manifest, redirects):
     :return:
     :rtype:
     """
+    redirecting = False
     for binding in redirects:
         for dep in manifest.dependentAssemblies:
             if match_binding_redirect(dep, binding):
                 logger.info("Redirecting %s version %s -> %s",
                             binding.name, dep.version, binding.newVersion)
                 dep.version = binding.newVersion
+                redirecting = True
+    return redirecting
 
 def checkCache(fnm, strip=False, upx=False, dist_nm=None):
     """
@@ -170,7 +191,15 @@ def checkCache(fnm, strip=False, upx=False, dist_nm=None):
         os.makedirs(cachedir)
     cacheindexfn = os.path.join(cachedir, "index.dat")
     if os.path.exists(cacheindexfn):
-        cache_index = load_py_data_struct(cacheindexfn)
+        try:
+            cache_index = load_py_data_struct(cacheindexfn)
+        except Exception as e:
+            # tell the user they may want to fix their cache
+            # .. however, don't delete it for them; if it keeps getting
+            #    corrupted, we'll never find out
+            logger.warn("pyinstaller bincache may be corrupted; "
+                        "use pyinstaller --clean to fix")
+            raise
     else:
         cache_index = {}
 
@@ -181,7 +210,12 @@ def checkCache(fnm, strip=False, upx=False, dist_nm=None):
         basenm = os.path.normcase(dist_nm)
     else:
         basenm = os.path.normcase(os.path.basename(fnm))
-    digest = cacheDigest(fnm)
+
+    # Binding redirects should be taken into account to see if the file
+    # needs to be reprocessed. The redirects may change if the versions of dependent
+    # manifests change due to system updates.
+    redirects = CONF.get('binding_redirects', [])
+    digest = cacheDigest(fnm, redirects)
     cachedfile = os.path.join(cachedir, basenm)
     cmd = None
     if basenm in cache_index:
@@ -194,7 +228,6 @@ def checkCache(fnm, strip=False, upx=False, dist_nm=None):
                 dylib.mac_set_relative_dylib_deps(cachedfile, dist_nm)
             return cachedfile
 
-    redirects = CONF.get('binding_redirects', [])
 
     # Optionally change manifest and its deps to private assemblies
     if fnm.lower().endswith(".manifest"):
@@ -287,7 +320,8 @@ def checkCache(fnm, strip=False, upx=False, dist_nm=None):
                             logger.error("From file %s", cachedfile, exc_info=1)
                         else:
                             # optionally change manifest to private assembly
-                            if CONF.get('win_private_assemblies', False):
+                            private = CONF.get('win_private_assemblies', False)
+                            if private:
                                 if manifest.publicKeyToken:
                                     logger.info("Changing %s into a private assembly",
                                                 os.path.basename(fnm))
@@ -298,14 +332,15 @@ def checkCache(fnm, strip=False, upx=False, dist_nm=None):
                                     # Exclude common-controls which is not bundled
                                     if dep.name != "Microsoft.Windows.Common-Controls":
                                         dep.publicKeyToken = None
-                            applyRedirects(manifest, redirects)
-                            try:
-                                manifest.update_resources(os.path.abspath(cachedfile),
-                                                          [name],
-                                                          [language])
-                            except Exception as e:
-                                logger.error(os.path.abspath(cachedfile))
-                                raise
+                            redirecting = applyRedirects(manifest, redirects)
+                            if redirecting or private:
+                                try:
+                                    manifest.update_resources(os.path.abspath(cachedfile),
+                                                              [name],
+                                                              [language])
+                                except Exception as e:
+                                    logger.error(os.path.abspath(cachedfile))
+                                    raise
 
     if cmd:
         try:
@@ -325,9 +360,17 @@ def checkCache(fnm, strip=False, upx=False, dist_nm=None):
     return cachedfile
 
 
-def cacheDigest(fnm):
-    data = open(fnm, "rb").read()
-    digest = hashlib.md5(data).digest()
+def cacheDigest(fnm, redirects):
+    hasher = hashlib.md5()
+    with open(fnm, "rb") as f:
+        for chunk in iter(lambda: f.read(16 * 1024), b""):
+            hasher.update(chunk)
+    if redirects:
+        redirects = str(redirects)
+        if is_py3:
+            redirects = redirects.encode('utf-8')
+        hasher.update(redirects)
+    digest = bytearray(hasher.digest())
     return digest
 
 
@@ -356,6 +399,20 @@ def _check_path_overlap(path):
     return True
 
 
+def _make_clean_directory(path):
+    """
+    Create a clean directory from the given directory name
+    """
+    if _check_path_overlap(path):
+        if os.path.isdir(path):
+            try:
+                os.remove(path)
+            except OSError:
+                _rmtree(path)
+
+        os.makedirs(path)
+
+
 def _rmtree(path):
     """
     Remove directory and all its contents, but only after user confirmation,
@@ -404,21 +461,14 @@ def format_binaries_and_datas(binaries_or_datas, workingdir=None):
     binaries_or_datas : list
         List of hook-style 2-tuples (e.g., the top-level `binaries` and `datas`
         attributes defined by hooks) whose:
-        * First element is either:
+        * The first element is either:
           * A glob matching only the absolute or relative paths of source
-            non-Python data files. The second element is then either:
-            * The relative path of the target directory into which these source
-              files will be recursively copied.
-            * The empty string, in which case these source files will be
-              recursively copied into the top-level target directory. (This is
-              usually _not_ what you want.)
+            non-Python data files.
           * The absolute or relative path of a source directory containing only
-            source non-Python data files. The second element is then either:
-            * The relative path of the target directory into which these source
-              files will be recursively copied.
-            * The empty string, in which case these source files will be
-              recursively copied into a new target subdirectory whose name is
-              this source directory's basename. (This is usually what you want.)
+            source non-Python data files.
+        * The second element ist he relative path of the target directory
+          into which these source files will be recursively copied.
+
         If the optional `workingdir` parameter is passed, source paths may be
         either absolute or relative; else, source paths _must_ be absolute.
     workingdir : str
@@ -438,6 +488,11 @@ def format_binaries_and_datas(binaries_or_datas, workingdir=None):
     toc_datas = set()
 
     for src_root_path_or_glob, trg_root_dir in binaries_or_datas:
+        if not trg_root_dir:
+            raise SystemExit("Empty DEST not allowed when adding binary "
+                             "and data files. "
+                             "Maybe you want to used %r.\nCaused by %r." %
+                             (os.curdir, src_root_path_or_glob))
         # Convert relative to absolute paths if required.
         if workingdir and not os.path.isabs(src_root_path_or_glob):
             src_root_path_or_glob = os.path.join(
@@ -445,15 +500,29 @@ def format_binaries_and_datas(binaries_or_datas, workingdir=None):
 
         # Normalize paths.
         src_root_path_or_glob = os.path.normpath(src_root_path_or_glob)
-
-        # List of the absolute paths of all source paths matching the
-        # current glob.
-        src_root_paths = glob.glob(src_root_path_or_glob)
+        if os.path.isfile(src_root_path_or_glob):
+            src_root_paths = [src_root_path_or_glob]
+        else:
+            # List of the absolute paths of all source paths matching the
+            # current glob.
+            src_root_paths = glob.glob(src_root_path_or_glob)
 
         if not src_root_paths:
-            raise FileNotFoundError(
-                'Path or glob "%s" not found or matches no files.' % (
-                src_root_path_or_glob))
+            msg = 'Unable to find "%s" when adding binary and data files.' % (
+                src_root_path_or_glob)
+            # on Debian/Ubuntu, missing pyconfig.h files can be fixed with
+            # installing python-dev
+            if src_root_path_or_glob.endswith("pyconfig.h"):
+                msg += """This would mean your Python installation doesn't
+come with proper library files. This usually happens by missing development
+package, or unsuitable build parameters of Python installation.
+* On Debian/Ubuntu, you would need to install Python development packages
+  * apt-get install python3-dev
+  * apt-get install python-dev
+* If you're building Python by yourself, please rebuild your Python with
+`--enable-shared` (or, `--enable-framework` on Darwin)
+"""
+            raise SystemExit(msg)
 
         for src_root_path in src_root_paths:
             if os.path.isfile(src_root_path):
@@ -464,11 +533,6 @@ def format_binaries_and_datas(binaries_or_datas, workingdir=None):
                         trg_root_dir, os.path.basename(src_root_path))),
                     os.path.normpath(src_root_path)))
             elif os.path.isdir(src_root_path):
-                # If no top-level target directory was passed, default this
-                # to the basename of the top-level source directory.
-                if not trg_root_dir:
-                    trg_root_dir = os.path.basename(src_root_path)
-
                 for src_dir, src_subdir_basenames, src_file_basenames in \
                     os.walk(src_root_path):
                     # Ensure the current source directory is a subdirectory
@@ -516,11 +580,27 @@ def _load_code(modname, filename):
         loader, portions = importer.find_loader(modname)
     else:
         loader = importer.find_module(modname)
-        portions = []
 
-    assert loader and hasattr(loader, 'get_code')
     logger.debug('Compiling %s', filename)
-    return loader.get_code(modname)
+    if loader and hasattr(loader, 'get_code'):
+        return loader.get_code(modname)
+    else:
+        # Just as ``python foo.bar`` will read and execute statements in
+        # ``foo.bar``,  even though it lacks the ``.py`` extension, so
+        # ``pyinstaller foo.bar``  should also work. However, Python's import
+        # machinery doesn't load files without a ``.py`` extension. So, use
+        # ``compile`` instead.
+        #
+        # On a side note, neither the Python 2 nor Python 3 calls to
+        # ``pkgutil`` and ``find_module`` above handle modules ending in
+        # ``.pyw``, even though ``imp.find_module`` and ``import <name>`` both
+        # work. This code supports ``.pyw`` files.
+
+        # Open the source file in binary mode and allow the `compile()` call to
+        # detect the source encoding.
+        with open_file(filename, 'rb') as f:
+            source = f.read()
+        return compile(source, filename, 'exec')
 
 def get_code_object(modname, filename):
     """
@@ -590,3 +670,31 @@ def strip_paths_in_code(co, new_filename=None):
                      co.co_varnames, new_filename, co.co_name,
                      co.co_firstlineno, co.co_lnotab,
                      co.co_freevars, co.co_cellvars)
+
+
+def fake_pyc_timestamp(buf):
+    """
+    Reset the timestamp from a .pyc-file header to a fixed value.
+
+    This enables deterministic builds without having to set pyinstaller
+    source metadata (mtime) since that changes the pyc-file contents.
+
+    _buf_ must at least contain the full pyc-file header.
+    """
+    assert buf[:4] == compat.BYTECODE_MAGIC, \
+        "Expected pyc magic {}, got {}".format(compat.BYTECODE_MAGIC, buf[:4])
+    start, end = 4, 8
+    if is_py37:
+        # see https://www.python.org/dev/peps/pep-0552/
+        (flags,) = struct.unpack_from(">I", buf, 4)
+        if flags & 1:
+            # We are in the future and hash-based pyc-files are used, so
+            # clear "check_source" flag, since there is no source
+            buf[4:8] = struct.pack(">I", flags ^ 2)
+            return buf
+        else:
+            # no hash-based pyc-file, timestamp is the next field
+            start, end = 8, 12
+
+    ts = b'pyi0'  # So people know where this comes from
+    return buf[:start] + ts + buf[end:]
