@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2013, PyInstaller Development Team.
+# Copyright (c) 2013-2018, PyInstaller Development Team.
 #
 # Distributed under the terms of the GNU General Public License with exception
 # for distributing bootloader.
@@ -8,13 +8,13 @@
 #-----------------------------------------------------------------------------
 
 
-from PyInstaller.hooks.hookutils import eval_statement
+from PyInstaller.utils.hooks import eval_statement
 
 hiddenimports = ["PyQt4.QtCore", "PyQt4.QtGui", "PyQt4.QtSvg"]
 
-if eval_statement("from PyQt4 import Qwt5; print hasattr(Qwt5, 'toNumpy')"):
+if eval_statement("from PyQt4 import Qwt5; print(hasattr(Qwt5, 'toNumpy'))"):
     hiddenimports.append("numpy")
-if eval_statement("from PyQt4 import Qwt5; print hasattr(Qwt5, 'toNumeric')"):
+if eval_statement("from PyQt4 import Qwt5; print(hasattr(Qwt5, 'toNumeric'))"):
     hiddenimports.append("Numeric")
-if eval_statement("from PyQt4 import Qwt5; print hasattr(Qwt5, 'toNumarray')"):
+if eval_statement("from PyQt4 import Qwt5; print(hasattr(Qwt5, 'toNumarray'))"):
     hiddenimports.append("numarray")
