@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2015-2016, PyInstaller Development Team.
+# Copyright (c) 2015-2018, PyInstaller Development Team.
 #
 # Distributed under the terms of the GNU General Public License with exception
 # for distributing bootloader.
@@ -11,4 +11,4 @@
 
 from PyInstaller.utils.hooks import collect_data_files
 
-datas = collect_data_files('appy.pod')
+datas = collect_data_files('appy.pod', True)
