@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2005-2016, PyInstaller Development Team.
+# Copyright (c) 2005-2018, PyInstaller Development Team.
 #
 # Distributed under the terms of the GNU General Public License with exception
 # for distributing bootloader.
@@ -8,7 +8,7 @@
 #-----------------------------------------------------------------------------
 
 """
-PEP-302 importers for frozen applications.
+PEP-302 and PEP-451 importers for frozen applications.
 """
 
 
@@ -67,7 +67,7 @@ else:
     def trace(msg, *a):
         pass
 
-# TODO Do we still need BuiltintImporter for Python 3 built-in modules?
+# Python 3 has it's own BuiltinImporter, we use this for Python2 only
 class BuiltinImporter(object):
     """
     PEP-302 wrapper of the built-in modules for sys.meta_path.
@@ -76,6 +76,7 @@ class BuiltinImporter(object):
     modules in the bundled ZIP archive.
     """
     def find_module(self, fullname, path=None):
+        # Deprecated in Python 3.4, see PEP-451
         imp_lock()
         module_loader = None  # None means - no module found by this importer.
 
@@ -87,6 +88,7 @@ class BuiltinImporter(object):
         return module_loader
 
     def load_module(self, fullname, path=None):
+        # Deprecated in Python 3.4, see PEP-451
         imp_lock()
 
         try:
@@ -150,12 +152,13 @@ class FrozenPackageImporter(object):
     This is called by FrozenImporter.find_module whenever a module is found as a result
     of searching module.__path__
     """
-    def __init__(self, importer, fullname):
-        self._fullname = fullname
+    def __init__(self, importer, entry_name):
+        self._entry_name = entry_name
         self._importer = importer
 
     def load_module(self, fullname):
-        return self._importer.load_module(fullname, self._fullname)
+        # Deprecated in Python 3.4, see PEP-451
+        return self._importer.load_module(fullname, self._entry_name)
 
 
 class FrozenImporter(object):
@@ -175,11 +178,16 @@ class FrozenImporter(object):
     class with method load_module(). Both these methods are implemented
     in one class.
 
+    This is also a PEP-451 finder and loader class for the ModuleSpec type
+    import system. A PEP-451 finder requires method find_spec(), a PEP-451
+    loader requires methods exec_module(), load_module(9 and (optionally)
+    create_module(). All these methods are implemented in this one class.
 
     To use this class just call
 
         FrozenImporter.install()
     """
+
     def __init__(self):
         """
         Load, unzip and initialize the Zip archive bundled with the executable.
@@ -226,7 +234,8 @@ class FrozenImporter(object):
 
     def __call__(self, path):
         """
-        PEP-302 sys.path_hook processor.
+        PEP-302 sys.path_hook processor. is_py2: This is only needed for Python
+        2; see comments at `path_hook installation`_.
 
         sys.path_hook is a list of callables, which will be checked in
         sequence to determine if they can handle a given path item.
@@ -241,15 +250,19 @@ class FrozenImporter(object):
 
 
     def find_module(self, fullname, path=None):
+        # Deprecated in Python 3.4, see PEP-451
         """
         PEP-302 finder.find_module() method for the ``sys.meta_path`` hook.
 
         fullname     fully qualified name of the module
-        path         None for a top-level module, or package.__path__ for submodules or subpackages.
+        path         None for a top-level module, or package.__path__
+                     for submodules or subpackages.
 
-        Return a loader object if the module was found, or None if it wasn't. If find_module() raises
-        an exception, it will be propagated to the caller, aborting the import.
+        Return a loader object if the module was found, or None if it wasn't.
+        If find_module() raises an exception, it will be propagated to the
+        caller, aborting the import.
         """
+
         # Acquire the interpreter's import lock for the current thread. This
         # lock should be used by import hooks to ensure thread-safety when
         # importing modules.
@@ -269,17 +282,17 @@ class FrozenImporter(object):
             modname = fullname.split('.')[-1]
 
             for p in path:
-                p = p.replace(SYS_PREFIX, "")
+                p = p[SYS_PREFIXLEN+1:]
                 parts = p.split(pyi_os_path.os_sep)
-                if not len(parts): continue
+                if not parts: continue
                 if not parts[0]:
                     parts = parts[1:]
                 parts.append(modname)
-                real_fullname = ".".join(parts)
-                if real_fullname in self.toc:
-                    module_loader = FrozenPackageImporter(self, real_fullname)
+                entry_name = ".".join(parts)
+                if entry_name in self.toc:
+                    module_loader = FrozenPackageImporter(self, entry_name)
                     trace("import %s as %s # PyInstaller PYZ (__path__ override: %s)",
-                          real_fullname, fullname, p)
+                          entry_name, fullname, p)
                     break
         # Release the interpreter's import lock.
         imp_unlock()
@@ -287,7 +300,8 @@ class FrozenImporter(object):
             trace("# %s not found in PYZ", fullname)
         return module_loader
 
-    def load_module(self, fullname, real_fullname=None):
+    def load_module(self, fullname, entry_name=None):
+        # Deprecated in Python 3.4, see PEP-451
         """
         PEP-302 loader.load_module() method for the ``sys.meta_path`` hook.
 
@@ -295,15 +309,15 @@ class FrozenImporter(object):
         an exception, preferably ImportError if an existing exception
         is not being propagated.
 
-        When called from FrozenPackageImporter, `real_fullname` is the name of the
+        When called from FrozenPackageImporter, `entry_name` is the name of the
         module as it is stored in the archive. This module will be loaded and installed
         into sys.modules using `fullname` as its name
         """
         # Acquire the interpreter's import lock.
         imp_lock()
         module = None
-        if real_fullname is None:
-            real_fullname=fullname
+        if entry_name is None:
+            entry_name = fullname
         try:
             # PEP302 If there is an existing module object named 'fullname'
             # in sys.modules, the loader must use that existing module.
@@ -312,7 +326,7 @@ class FrozenImporter(object):
             # Module not in sys.modules - load it and it to sys.modules.
             if module is None:
                 # Load code object from the bundled ZIP archive.
-                is_pkg, bytecode = self._pyz_archive.extract(real_fullname)
+                is_pkg, bytecode = self._pyz_archive.extract(entry_name)
                 # Create new empty 'module' object.
                 module = imp_new_module(fullname)
 
@@ -320,17 +334,9 @@ class FrozenImporter(object):
                 # e.g. /absolute/path/frozen_executable/path/to/module/module_name.pyc
                 # Paths from developer machine are masked.
 
-                ### Set __file__ attribute of a module relative to the executable
-                # so that data files can be found. The absolute absolute path
-                # to the executable is taken from sys.prefix. In onefile mode it
-                # points to the temp directory where files are unpacked by PyInstaller.
-                # Then, append the appropriate suffix (__init__.pyc for a package, or just .pyc for a module).
-                if is_pkg:
-                    module.__file__ = pyi_os_path.os_path_join(pyi_os_path.os_path_join(SYS_PREFIX,
-                        fullname.replace('.', pyi_os_path.os_sep)), '__init__.pyc')
-                else:
-                    module.__file__ = pyi_os_path.os_path_join(SYS_PREFIX,
-                        fullname.replace('.', pyi_os_path.os_sep) + '.pyc')
+                # Set __file__ attribute of a module relative to the
+                # executable so that data files can be found.
+                module.__file__ = self.get_filename(entry_name)
 
                 ### Set __path__  if 'fullname' is a package.
                 # Python has modules and packages. A Python package is container
@@ -375,7 +381,7 @@ class FrozenImporter(object):
                 # consolidate all module attributes.
                 if sys.version_info[0:2] > (3, 3):
                     module.__spec__ = _frozen_importlib.ModuleSpec(
-                        real_fullname, self, is_package=is_pkg)
+                        entry_name, self, is_package=is_pkg)
 
                 ### Add module object to sys.modules dictionary.
                 # Module object must be in sys.modules before the loader
@@ -411,13 +417,9 @@ class FrozenImporter(object):
     ### Optional Extensions to the PEP-302 Importer Protocol
 
     def is_package(self, fullname):
-        """
-        Return always False since built-in modules are never packages.
-        """
         if fullname in self.toc:
             try:
-                is_pkg, bytecode = self._pyz_archive.extract(fullname)
-                return bool(is_pkg)
+                return self._pyz_archive.is_package(fullname)
             except Exception:
                 raise ImportError('Loader FrozenImporter cannot handle module ' + fullname)
         else:
@@ -429,13 +431,12 @@ class FrozenImporter(object):
 
         ImportError should be raised if module not found.
         """
-        if fullname in self.toc:
-            try:
-                is_pkg, bytecode = self._pyz_archive.extract(fullname)
-                return bytecode
-            except Exception:
-                raise ImportError('Loader FrozenImporter cannot handle module ' + fullname)
-        else:
+        try:
+            # extract() returns None if fullname not in the archive, thus the
+            # next line will raise an execpion which will be catched just
+            # below and raise the ImportError.
+            return self._pyz_archive.extract(fullname)[1]
+        except:
             raise ImportError('Loader FrozenImporter cannot handle module ' + fullname)
 
     def get_source(self, fullname):
@@ -463,7 +464,7 @@ class FrozenImporter(object):
         module.__file__ (or pkg.__path__ items)
         """
         assert path.startswith(SYS_PREFIX + pyi_os_path.os_sep)
-        fullname = path[len(SYS_PREFIX)+1:]
+        fullname = path[SYS_PREFIXLEN+1:]
         if fullname in self.toc:
             # If the file is in the archive, return this
             return self._pyz_archive.extract(fullname)[1]
@@ -474,14 +475,16 @@ class FrozenImporter(object):
             with open(path, 'rb') as fp:
                 return fp.read()
 
-    # TODO Do we really need to implement this method?
     def get_filename(self, fullname):
         """
         This method should return the value that __file__ would be set to
         if the named module was loaded. If the module is not found, then
         ImportError should be raised.
         """
-        # Then, append the appropriate suffix (__init__.pyc for a package, or just .pyc for a module).
+        # The absolute absolute path to the executable is taken from
+        # sys.prefix. In onefile mode it points to the temp directory where
+        # files are unpacked by PyInstaller. Then, append the appropriate
+        # suffix (__init__.pyc for a package, or just .pyc for a module).
         # Method is_package() will raise ImportError if module not found.
         if self.is_package(fullname):
             filename = pyi_os_path.os_path_join(pyi_os_path.os_path_join(SYS_PREFIX,
@@ -491,7 +494,140 @@ class FrozenImporter(object):
                 fullname.replace('.', pyi_os_path.os_sep) + '.pyc')
         return filename
 
+    def find_spec(self, fullname, path=None, target=None):
+        """
+        PEP-451 finder.find_spec() method for the ``sys.meta_path`` hook.
 
+        fullname     fully qualified name of the module
+        path         None for a top-level module, or package.__path__ for
+                     submodules or subpackages.
+        target       unused by this Finder
+
+        Finders are still responsible for identifying, and typically creating,
+        the loader that should be used to load a module. That loader will now
+        be stored in the module spec returned by find_spec() rather than
+        returned directly. As is currently the case without the PEP-452, if a
+        loader would be costly to create, that loader can be designed to defer
+        the cost until later.
+
+        Finders must return ModuleSpec objects when find_spec() is called.
+        This new method replaces find_module() and find_loader() (in the
+        PathEntryFinder case). If a loader does not have find_spec(),
+        find_module() and find_loader() are used instead, for
+        backward-compatibility.
+        """
+        entry_name = None  # None means - no module found in this importer.
+
+        if fullname in self.toc:
+            entry_name = fullname
+            trace("import %s # PyInstaller PYZ", fullname)
+        elif path is not None:
+            # Try to handle module.__path__ modifications by the modules themselves
+            # Reverse the fake __path__ we added to the package module to a
+            # dotted module name and add the tail module from fullname onto that
+            # to synthesize a new fullname
+            modname = fullname.rsplit('.')[-1]
+
+            for p in path:
+                p = p[SYS_PREFIXLEN+1:]
+                parts = p.split(pyi_os_path.os_sep)
+                if not parts: continue
+                if not parts[0]:
+                    parts = parts[1:]
+                parts.append(modname)
+                entry_name = ".".join(parts)
+                if entry_name in self.toc:
+                    trace("import %s as %s # PyInstaller PYZ (__path__ override: %s)",
+                          entry_name, fullname, p)
+                    break
+            else:
+                entry_name = None
+
+        if entry_name is None:
+            trace("# %s not found in PYZ", fullname)
+            return None
+
+        # origin has to be the filename
+        origin = self.get_filename(entry_name)
+        is_pkg = self.is_package(entry_name)
+
+        spec =  _frozen_importlib.ModuleSpec(
+            fullname, self,
+            is_package=is_pkg, origin=origin,
+            # Provide the entry_name for the loader to use during loading
+            loader_state = entry_name)
+
+        # Make the import machinery set __file__.
+        # PEP 451 says: "has_location" is true if the module is locatable. In
+        # that case the spec's origin is used as the location and __file__ is
+        # set to spec.origin. If additional location information is required
+        # (e.g. zipimport), that information may be stored in
+        # spec.loader_state.
+        spec.has_location = True
+        return spec
+
+    def create_module(self, spec):
+        """
+        PEP-451 loader.create_module() method for the ``sys.meta_path`` hook.
+
+        Loaders may also implement create_module() that will return a new
+        module to exec. It may return None to indicate that the default module
+        creation code should be used. One use case, though atypical, for
+        create_module() is to provide a module that is a subclass of the
+        builtin module type. Most loaders will not need to implement
+        create_module(),
+
+        create_module() should properly handle the case where it is called
+        more than once for the same spec/module. This may include returning
+        None or raising ImportError.
+        """
+        # Opposed to what is defined in PEP-451, this method is not optional.
+        # We want the default results, so we simply return None (which is
+        # handled for su my the import machinery). See
+        # https://bugs.python.org/issue23014 for more information.
+        return None
+
+    def exec_module(self, module):
+        """
+        PEP-451 loader.exec_module() method for the ``sys.meta_path`` hook.
+
+        Loaders will have a new method, exec_module(). Its only job is to
+        "exec" the module and consequently populate the module's namespace. It
+        is not responsible for creating or preparing the module object, nor
+        for any cleanup afterward. It has no return value. exec_module() will
+        be used during both loading and reloading.
+
+        exec_module() should properly handle the case where it is called more
+        than once. For some kinds of modules this may mean raising ImportError
+        every time after the first time the method is called. This is
+        particularly relevant for reloading, where some kinds of modules do
+        not support in-place reloading.
+        """
+        spec = module.__spec__
+        bytecode = self.get_code(spec.loader_state)
+
+        # Set by the import machinery
+        assert hasattr(module, '__file__')
+
+        # If `submodule_search_locations` is not None, this is a package;
+        # set __path__.
+        if spec.submodule_search_locations is not None:
+            # Since PYTHONHOME is set in bootloader, 'sys.prefix' points to
+            # the correct path where PyInstaller should find bundled dynamic
+            # libraries. In one-file mode it points to the tmp directory where
+            # bundled files are extracted at execution time.
+            #
+            # __path__ cannot be empty list because 'wx' module prepends
+            # something to it. It cannot contain value 'sys.prefix' because
+            # 'xml.etree.cElementTree' fails otherwise.
+            #
+            # Set __path__ to point to 'sys.prefix/package/subpackage'.
+            module.__path__ = [pyi_os_path.os_path_dirname(module.__file__)]
+
+        exec(bytecode, module.__dict__)
+
+
+# is_py2: This is only needed for Python 2.
 class CExtensionImporter(object):
     """
     PEP-302 hook for sys.meta_path to load Python C extension modules.
@@ -510,6 +646,7 @@ class CExtensionImporter(object):
         self._file_cache = set(files)
 
     def find_module(self, fullname, path=None):
+        # Deprecated in Python 3.4, see PEP-451
         imp_lock()
         module_loader = None  # None means - no module found by this importer.
 
@@ -523,6 +660,7 @@ class CExtensionImporter(object):
         return module_loader
 
     def load_module(self, fullname, path=None):
+        # Deprecated in Python 3.4, see PEP-451
         imp_lock()
 
         module = None
@@ -542,8 +680,8 @@ class CExtensionImporter(object):
                         if filename in self._file_cache:
                             break
                     filename = pyi_os_path.os_path_join(SYS_PREFIX, filename)
-                    fp = open(filename, 'rb')
-                    module = imp.load_module(fullname, fp, filename, ext_tuple)
+                    with open(filename, 'rb') as fp:
+                        module = imp.load_module(fullname, fp, filename, ext_tuple)
                     # Set __file__ attribute.
                     if hasattr(module, '__setattr__'):
                         module.__file__ = filename
@@ -570,7 +708,10 @@ class CExtensionImporter(object):
                             continue
                         # Load module.
                         loader = EXTENSION_LOADER(fullname, filename)
-                        module = loader.load_module(fullname)
+                        try:
+                            module = loader.load_module(fullname)
+                        except ImportError as e:
+                            raise ImportError('%s: %s' % (e, fullname))
 
         except Exception:
             # Remove 'fullname' from sys.modules if it was appended there.
@@ -619,12 +760,9 @@ class CExtensionImporter(object):
         module.__file__ (or pkg.__path__ items)
         """
         # Since __file__ attribute works properly just try to open and read it.
-        fp = open(path, 'rb')
-        content = fp.read()
-        fp.close()
-        return content
+        with open(path, 'rb') as fp:
+            return fp.read()
 
-    # TODO Do we really need to implement this method?
     def get_filename(self, fullname):
         """
         This method should return the value that __file__ would be set to
@@ -663,15 +801,26 @@ def install():
     # other places.
     fimp = FrozenImporter()
     sys.meta_path.append(fimp)
-    # Add the FrozenImporter to `sys.path_hook`, too, since
-    # `pkgutil.get_loader()` does not use `sys.meta_path`. See issue
-    # #1689.
-    sys.path_hooks.append(fimp)
 
-    # Import hook for the C extension modules.
-    sys.meta_path.append(CExtensionImporter())
+    if sys.version_info[0] == 2:
+        # _`path_hook installation`: Add the FrozenImporter to `sys.path_hook`,
+        # too, since `pkgutil.get_loader()` does not use `sys.meta_path`. See
+        # issue #1689.
+        sys.path_hooks.append(fimp)
 
-    if sys.version_info[0] > 2:
+        # Import hook for renamed C extension modules. The path hook above means
+        # that the standard Python import system for C extensions will only work
+        # if those extensions are not in a submodule. In particular: A C
+        # extension such as ``foo.bar`` hasn't been loaded. ``sys.meta_path`` is
+        # checked, but provides no loader. ``sys.path_hooks`` is checked, and
+        # claims that it loads anything in ``foo``. Therefore, the default
+        # Python import mechanism will not be invoked, per `PEP 302
+        # <https://www.python.org/dev/peps/pep-0302/#id28>`_. This casues the
+        # import to fail, since the ``FrozenImporter`` class doesn't load C
+        # extensions.
+        sys.meta_path.append(CExtensionImporter())
+
+    else:
         # On Windows there is importer _frozen_importlib.WindowsRegistryFinder that
         # looks for Python modules in Windows registry. The frozen executable should
         # not look for anything in the Windows registry. Remove this importer from
@@ -680,11 +829,18 @@ def install():
             if hasattr(item, '__name__') and item.__name__ == 'WindowsRegistryFinder':
                 sys.meta_path.remove(item)
                 break
-        # _frozen_importlib.PathFinder is in Python 3 the last importer on sys.meta_path.
-        # This importer is also able handle Python C extensions. However, PyInstaller
-        # needs own importer to allow extension name 'module.submodle.so'.
-        # Add the pathfinder at the end of sys.meta_path.
-        pf_idx = 2  # PathFinder is the 3rd in sys.meta_path.
-        pf = sys.meta_path.pop(pf_idx)
-        sys.meta_path.append(pf)
+        # _frozen_importlib.PathFinder is also able to handle Python C
+        # extensions. However, PyInstaller needs its own importer since it
+        # uses extension names like 'module.submodle.so' (instead of paths).
+        # As of Python 3.7.0b2, there are several PathFinder instances (and
+        # duplicate ones) on sys.meta_path. This propobly is a bug, see
+        # https://bugs.python.org/issue33128. Thus we need to move all of them
+        # to the end, eliminating duplicates .
+        pathFinders = []
+        for item in reversed(sys.meta_path):
+            if getattr(item, '__name__', None) == 'PathFinder':
+                sys.meta_path.remove(item)
+                if not item in pathFinders:
+                    pathFinders.append(item)
+        sys.meta_path.extend(reversed(pathFinders))
         # TODO Do we need for Python 3 _frozen_importlib.FrozenImporter? Could it be also removed?
