@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2013-2016, PyInstaller Development Team.
+# Copyright (c) 2013-2018, PyInstaller Development Team.
 #
 # Distributed under the terms of the GNU General Public License with exception
 # for distributing bootloader.
@@ -12,11 +12,11 @@
 #
 # Currently not implemented in the Manifest class:
 # * Validation (only very basic sanity checks are currently in place)
-# * comClass, typelib, comInterfaceProxyStub and windowClass child elements of 
+# * comClass, typelib, comInterfaceProxyStub and windowClass child elements of
 #   the file element
-# * comInterfaceExternalProxyStub and windowClass child elements of the 
+# * comInterfaceExternalProxyStub and windowClass child elements of the
 #   assembly element
-# * Application Configuration File and Multilanguage User Interface (MUI) 
+# * Application Configuration File and Multilanguage User Interface (MUI)
 #   support when searching for assembly files
 #
 # Isolated Applications and Side-by-side Assemblies:
@@ -25,14 +25,14 @@
 # Changelog:
 # 2009-12-17  fix: small glitch in toxml / toprettyxml methods (xml declaration
 #                  wasn't replaced when a different encodig than UTF-8 was used)
-#             chg: catch xml.parsers.expat.ExpatError and re-raise as 
+#             chg: catch xml.parsers.expat.ExpatError and re-raise as
 #                  ManifestXMLParseError
 #             chg: support initialize option in parse method also
 #
 # 2009-12-13  fix: fixed os import
 #             fix: skip invalid / empty dependent assemblies
 #
-# 2009-08-21  fix: Corrected assembly searching sequence for localized 
+# 2009-08-21  fix: Corrected assembly searching sequence for localized
 #                  assemblies
 #             fix: Allow assemblies with no dependent files
 #
@@ -42,11 +42,11 @@
 #
 # 2009-07-30  fix: Potential failure in File.calc_hash method if hash
 #                  algorythm not supported
-#             add: Publisher configuration (policy) support when searching for 
+#             add: Publisher configuration (policy) support when searching for
 #                  assembly files
 #             fix: Private assemblies are now actually found if present (and no
 #                  shared assembly exists)
-#             add: Python 2.3 compatibility (oldest version supported by 
+#             add: Python 2.3 compatibility (oldest version supported by
 #                  pyinstaller)
 #
 # 2009-07-28  chg: Code cleanup, removed a bit of redundancy
@@ -56,14 +56,14 @@
 # 2009-06-18  chg: Use glob instead of regular expression in Manifest.find_files
 #
 # 2009-05-04  fix: Don't fail if manifest has empty description
-#             fix: Manifests created by the toxml, toprettyxml, writexml or 
-#                  writeprettyxml methods are now correctly recognized by 
+#             fix: Manifests created by the toxml, toprettyxml, writexml or
+#                  writeprettyxml methods are now correctly recognized by
 #                  Windows, which expects the XML declaration to be ordered
 #                  version-encoding-standalone (standalone being optional)
-#             add: 'encoding' keyword argument in toxml, toprettyxml, writexml 
+#             add: 'encoding' keyword argument in toxml, toprettyxml, writexml
 #                  and writeprettyxml methods
-#             chg: UpdateManifestResourcesFromXML and 
-#                  UpdateManifestResourcesFromXMLFile: set resource name 
+#             chg: UpdateManifestResourcesFromXML and
+#                  UpdateManifestResourcesFromXMLFile: set resource name
 #                  depending on file type ie. exe or dll
 #             fix: typo in __main__: UpdateManifestResourcesFromDataFile
 #                  should have been UpdateManifestResourcesFromXMLFile
@@ -73,7 +73,7 @@
 
 """
 Create, parse and write MS Windows Manifest files.
-Find files which are part of an assembly, by searching shared and 
+Find files which are part of an assembly, by searching shared and
 private assemblies.
 Update or add manifest resources in Win32 PE files.
 
@@ -92,20 +92,12 @@ from xml.dom import Node, minidom
 from xml.dom.minidom import Document, Element
 
 from PyInstaller import compat
-from PyInstaller.compat import architecture, is_py2
+from PyInstaller.compat import architecture, string_types
 from PyInstaller import log as logging
 from PyInstaller.utils.win32 import winresource
 
 logger = logging.getLogger(__name__)
 
-# String types to replace `isinstance(foo, str)`
-# Use `isinstance(foo, string_types)` instead.
-
-if is_py2:
-    string_types = basestring
-else:
-    string_types = str
-
 LANGUAGE_NEUTRAL_NT5 = "x-ww"
 LANGUAGE_NEUTRAL_NT6 = "none"
 RT_MANIFEST = 24
@@ -155,13 +147,13 @@ if winresource:
 else:
     _File = _Dummy
 
-    
+
 class File(_File):
 
     """ A file referenced by an assembly inside a manifest. """
-    
-    def __init__(self, filename="", hashalg=None, hash=None, comClasses=None, 
-                 typelibs=None, comInterfaceProxyStubs=None, 
+
+    def __init__(self, filename="", hashalg=None, hash=None, comClasses=None,
+                 typelibs=None, comInterfaceProxyStubs=None,
                  windowClasses=None):
         if winresource:
             winresource.File.__init__(self, filename)
@@ -185,15 +177,14 @@ class File(_File):
     def calc_hash(self, hashalg=None):
         """
         Calculate the hash of the file.
-        
-        Will be called automatically from the constructor if the file exists 
-        and hashalg is given (and supported), but may also be called manually 
+
+        Will be called automatically from the constructor if the file exists
+        and hashalg is given (and supported), but may also be called manually
         e.g. to update the hash if the file has changed.
-        
+
         """
-        fd = open(self.filename, "rb")
-        buf = fd.read()
-        fd.close()
+        with open(self.filename, "rb") as fd:
+            buf = fd.read()
         if hashalg:
             self.hashalg = hashalg.upper()
         self.hash = getattr(hashlib, self.hashalg.lower())(buf).hexdigest()
@@ -205,7 +196,7 @@ class File(_File):
             logger.info("Found file %s", fn)
             return fn
         else:
-            logger.warn("No such file %s", fn)
+            logger.warning("No such file %s", fn)
             return None
 
 
@@ -218,30 +209,30 @@ class ManifestXMLParseError(InvalidManifestError):
 
 
 class Manifest(object):
-    
+
     # Manifests:
     # http://msdn.microsoft.com/en-us/library/aa375365%28VS.85%29.aspx
 
     """
     Manifest constructor.
-    
+
     To build a basic manifest for your application:
-      mf = Manifest(type='win32', name='YourAppName', language='*', 
+      mf = Manifest(type='win32', name='YourAppName', language='*',
                     processorArchitecture='x86', version=[1, 0, 0, 0])
-    
+
     To write the XML to a manifest file:
       mf.writexml("YourAppName.exe.manifest")
     or
       mf.writeprettyxml("YourAppName.exe.manifest")
-    
+
     """
 
-    def __init__(self, manifestVersion=None, noInheritable=False, 
-                 noInherit=False, type_=None, name=None, language=None, 
-                 processorArchitecture=None, version=None, 
-                 publicKeyToken=None, description=None, 
-                 requestedExecutionLevel=None, uiAccess=None, 
-                 dependentAssemblies=None, files=None, 
+    def __init__(self, manifestVersion=None, noInheritable=False,
+                 noInherit=False, type_=None, name=None, language=None,
+                 processorArchitecture=None, version=None,
+                 publicKeyToken=None, description=None,
+                 requestedExecutionLevel=None, uiAccess=None,
+                 dependentAssemblies=None, files=None,
                  comInterfaceExternalProxyStubs=None):
         self.filename = None
         self.optional = None
@@ -256,9 +247,9 @@ class Manifest(object):
         self.version = version
         self.publicKeyToken = publicKeyToken
         # publicKeyToken:
-        # A 16-character hexadecimal string that represents the last 8 bytes 
-        # of the SHA-1 hash of the public key under which the assembly is 
-        # signed. The public key used to sign the catalog must be 2048 bits 
+        # A 16-character hexadecimal string that represents the last 8 bytes
+        # of the SHA-1 hash of the public key under which the assembly is
+        # signed. The public key used to sign the catalog must be 2048 bits
         # or greater. Required for all shared side-by-side assemblies.
         # http://msdn.microsoft.com/en-us/library/aa375692(VS.85).aspx
         self.applyPublisherPolicy = None
@@ -284,33 +275,33 @@ class Manifest(object):
         return repr(self.toxml())
 
     def add_dependent_assembly(self, manifestVersion=None, noInheritable=False,
-                 noInherit=False, type_=None, name=None, language=None, 
-                 processorArchitecture=None, version=None, 
-                 publicKeyToken=None, description=None, 
-                 requestedExecutionLevel=None, uiAccess=None, 
-                 dependentAssemblies=None, files=None, 
+                 noInherit=False, type_=None, name=None, language=None,
+                 processorArchitecture=None, version=None,
+                 publicKeyToken=None, description=None,
+                 requestedExecutionLevel=None, uiAccess=None,
+                 dependentAssemblies=None, files=None,
                  comInterfaceExternalProxyStubs=None):
         """
         Shortcut for self.dependentAssemblies.append(Manifest(*args, **kwargs))
         """
-        self.dependentAssemblies.append(Manifest(manifestVersion, 
-                                        noInheritable, noInherit, type_, name, 
-                                        language, processorArchitecture, 
-                                        version, publicKeyToken, description, 
-                                        requestedExecutionLevel, uiAccess, 
-                                        dependentAssemblies, files, 
+        self.dependentAssemblies.append(Manifest(manifestVersion,
+                                        noInheritable, noInherit, type_, name,
+                                        language, processorArchitecture,
+                                        version, publicKeyToken, description,
+                                        requestedExecutionLevel, uiAccess,
+                                        dependentAssemblies, files,
                                         comInterfaceExternalProxyStubs))
         if self.filename:
             # Enable search for private assembly by assigning bogus filename
             # (only the directory has to be correct)
-            self.dependentAssemblies[-1].filename = ":".join((self.filename, 
+            self.dependentAssemblies[-1].filename = ":".join((self.filename,
                                                               name))
-    
-    def add_file(self, name="", hashalg="", hash="", comClasses=None, 
-                 typelibs=None, comInterfaceProxyStubs=None, 
+
+    def add_file(self, name="", hashalg="", hash="", comClasses=None,
+                 typelibs=None, comInterfaceProxyStubs=None,
                  windowClasses=None):
         """ Shortcut for manifest.files.append """
-        self.files.append(File(name, hashalg, hash, comClasses, 
+        self.files.append(File(name, hashalg, hash, comClasses,
                           typelibs, comInterfaceProxyStubs, windowClasses))
 
     @classmethod
@@ -321,10 +312,10 @@ class Manifest(object):
     def get_manifest_dir(cls):
         winsxs = cls.get_winsxs_dir()
         if not os.path.isdir(winsxs):
-            logger.warn("No such dir %s", winsxs)
+            logger.warning("No such dir %s", winsxs)
         manifests = os.path.join(winsxs, "Manifests")
         if not os.path.isdir(manifests):
-            logger.warn("No such dir %s", manifests)
+            logger.warning("No such dir %s", manifests)
         return manifests
 
     @classmethod
@@ -334,7 +325,7 @@ class Manifest(object):
             # Windows XP
             pcfiles = os.path.join(winsxs, "Policies")
             if not os.path.isdir(pcfiles):
-                logger.warn("No such dir %s", pcfiles)
+                logger.warning("No such dir %s", pcfiles)
         else:
             # Vista or later
             pcfiles = cls.get_manifest_dir()
@@ -380,7 +371,7 @@ class Manifest(object):
                                         ".manifest")
             for manifestpth in glob(policies):
                 if not os.path.isfile(manifestpth):
-                    logger.warn("Not a file %s", manifestpth)
+                    logger.warning("Not a file %s", manifestpth)
                     continue
                 logger.info("Found %s", manifestpth)
                 try:
@@ -425,7 +416,7 @@ class Manifest(object):
 
     def find_files(self, ignore_policies=True):
         """ Search shared and private assemblies and return a list of files.
-        
+
         If any files are not found, return an empty list.
 
         IMPORTANT NOTE: On some Windows systems, the dependency listed in the manifest
@@ -439,9 +430,9 @@ class Manifest(object):
         This is important because Python 2.7's app manifest depends on version 21022
         of the VC90 assembly, but the Python 2.7.9 installer will install version
         30729 of the assembly along with a policy file that enacts the version redirect.
-        
+
         """
-        
+
         # Shared Assemblies:
         # http://msdn.microsoft.com/en-us/library/aa375996%28VS.85%29.aspx
         #
@@ -453,9 +444,9 @@ class Manifest(object):
         #
         # NOTE:
         # Multilanguage User Interface (MUI) support not yet implemented
-        
+
         files = []
-        
+
         languages = []
         if self.language not in (None, "", "*", "neutral"):
             languages.append(self.getlanguage())
@@ -468,50 +459,50 @@ class Manifest(object):
             if self.language != "en":
                 languages.append("en")
         languages.append(self.getlanguage("*"))
-        
+
         manifests = self.get_manifest_dir()
         winsxs = self.get_winsxs_dir()
 
         for language in languages:
             version = self.version
-            
+
             # Search for publisher configuration
             if not ignore_policies and version:
                 version = self.get_policy_redirect(language, version)
-            
+
             # Search for assemblies according to assembly searching sequence
             paths = []
             if os.path.isdir(manifests):
                 # Add winsxs search paths
                 # Search for manifests in Windows\WinSxS\Manifests
-                paths.extend(glob(os.path.join(manifests, 
-                                               self.getid(language=language, 
-                                                          version=version) + 
+                paths.extend(glob(os.path.join(manifests,
+                                               self.getid(language=language,
+                                                          version=version) +
                                                "_*.manifest")))
             if self.filename:
                 # Add private assembly search paths
                 # Search for manifests inside assembly folders that are in
                 # the same folder as the depending manifest.
                 dirnm = os.path.dirname(self.filename)
-                if language in (LANGUAGE_NEUTRAL_NT5, 
+                if language in (LANGUAGE_NEUTRAL_NT5,
                                 LANGUAGE_NEUTRAL_NT6):
                     for ext in (".dll", ".manifest"):
                         paths.extend(glob(os.path.join(dirnm, self.name + ext)))
-                        paths.extend(glob(os.path.join(dirnm, self.name, 
+                        paths.extend(glob(os.path.join(dirnm, self.name,
                                                        self.name + ext)))
                 else:
                     for ext in (".dll", ".manifest"):
-                        paths.extend(glob(os.path.join(dirnm, language, 
+                        paths.extend(glob(os.path.join(dirnm, language,
                                                        self.name + ext)))
                     for ext in (".dll", ".manifest"):
-                        paths.extend(glob(os.path.join(dirnm, language, 
-                                                       self.name, 
+                        paths.extend(glob(os.path.join(dirnm, language,
+                                                       self.name,
                                                        self.name + ext)))
             logger.info("Searching for assembly %s ...",
                         self.getid(language=language, version=version))
             for manifestpth in paths:
                 if not os.path.isfile(manifestpth):
-                    logger.warn("Not a file %s", manifestpth)
+                    logger.warning("Not a file %s", manifestpth)
                     continue
                 assemblynm = os.path.basename(
                     os.path.splitext(manifestpth)[0])
@@ -531,8 +522,8 @@ class Manifest(object):
                         # dir is in Windows\WinSxS
                         assemblydir = os.path.join(winsxs, assemblynm)
                         if not os.path.isdir(assemblydir):
-                            logger.warn("No such dir %s", assemblydir)
-                            logger.warn("Assembly incomplete")
+                            logger.warning("No such dir %s", assemblydir)
+                            logger.warning("Assembly incomplete")
                             return []
                     else:
                         # Manifest is inside assembly dir.
@@ -545,28 +536,28 @@ class Manifest(object):
                         else:
                             # If any of our files does not exist,
                             # the assembly is incomplete
-                            logger.warn("Assembly incomplete")
+                            logger.warning("Assembly incomplete")
                             return []
                 return files
 
-        logger.warn("Assembly not found")
+        logger.warning("Assembly not found")
         return []
 
     def getid(self, language=None, version=None):
         """
         Return an identification string which uniquely names a manifest.
 
-        This string is a combination of the manifest's processorArchitecture, 
+        This string is a combination of the manifest's processorArchitecture,
         name, publicKeyToken, version and language.
-        
+
         Arguments:
-        version (tuple or list of integers) - If version is given, use it 
-                                              instead of the manifest's 
+        version (tuple or list of integers) - If version is given, use it
+                                              instead of the manifest's
                                               version.
-        
+
         """
         if not self.name:
-            logger.warn("Assembly metadata incomplete")
+            logger.warning("Assembly metadata incomplete")
             return ""
         id = []
         if self.processorArchitecture:
@@ -581,40 +572,40 @@ class Manifest(object):
         if language:
             id.append(language)
         return "_".join(id)
-    
+
     def getlanguage(self, language=None, windowsversion=None):
         """
         Get and return the manifest's language as string.
-        
-        Can be either language-culture e.g. 'en-us' or a string indicating 
-        language neutrality, e.g. 'x-ww' on Windows XP or 'none' on Vista 
+
+        Can be either language-culture e.g. 'en-us' or a string indicating
+        language neutrality, e.g. 'x-ww' on Windows XP or 'none' on Vista
         and later.
-        
+
         """
         if not language:
             language = self.language
         if language in (None, "", "*", "neutral"):
             return (LANGUAGE_NEUTRAL_NT5,
-                    LANGUAGE_NEUTRAL_NT6)[(windowsversion or 
+                    LANGUAGE_NEUTRAL_NT6)[(windowsversion or
                                            sys.getwindowsversion()) >= (6, )]
         return language
-    
+
     def getpolicyid(self, fuzzy=True, language=None, windowsversion=None):
         """
         Return an identification string which can be used to find a policy.
 
-        This string is a combination of the manifest's processorArchitecture, 
+        This string is a combination of the manifest's processorArchitecture,
         major and minor version, name, publicKeyToken and language.
-        
+
         Arguments:
-        fuzzy (boolean)             - If False, insert the full version in 
+        fuzzy (boolean)             - If False, insert the full version in
                                       the id string. Default is True (omit).
-        windowsversion              - If not specified (or None), default to 
+        windowsversion              - If not specified (or None), default to
         (tuple or list of integers)   sys.getwindowsversion().
-        
+
         """
         if not self.name:
-            logger.warn("Assembly metadata incomplete")
+            logger.warning("Assembly metadata incomplete")
             return ""
         id = []
         if self.processorArchitecture:
@@ -651,24 +642,24 @@ class Manifest(object):
     def load_dom(self, domtree, initialize=True):
         """
         Load manifest from DOM tree.
-        
+
         If initialize is True (default), reset existing attributes first.
-        
+
         """
         if domtree.nodeType == Node.DOCUMENT_NODE:
             rootElement = domtree.documentElement
         elif domtree.nodeType == Node.ELEMENT_NODE:
             rootElement = domtree
         else:
-            raise InvalidManifestError("Invalid root element node type " + 
-                                       str(rootElement.nodeType) + 
+            raise InvalidManifestError("Invalid root element node type " +
+                                       str(rootElement.nodeType) +
                                        " - has to be one of (DOCUMENT_NODE, "
                                        "ELEMENT_NODE)")
-        allowed_names = ("assembly", "assemblyBinding", "configuration", 
+        allowed_names = ("assembly", "assemblyBinding", "configuration",
                          "dependentAssembly")
         if rootElement.tagName not in allowed_names:
             raise InvalidManifestError(
-                "Invalid root element <%s> - has to be one of <%s>" % 
+                "Invalid root element <%s> - has to be one of <%s>" %
                 (rootElement.tagName, ">, <".join(allowed_names)))
         # logger.info("loading manifest metadata from element <%s>", rootElement.tagName)
         if rootElement.tagName == "configuration":
@@ -679,9 +670,9 @@ class Manifest(object):
             if initialize:
                 self.__init__()
             self.manifestType = rootElement.tagName
-            self.manifestVersion = [int(i) 
-                                    for i in 
-                                    (rootElement.getA("manifestVersion") or 
+            self.manifestVersion = [int(i)
+                                    for i in
+                                    (rootElement.getA("manifestVersion") or
                                      "1.0").split(".")]
             self.noInheritable = bool(rootElement.getFCEByTN("noInheritable"))
             self.noInherit = bool(rootElement.getFCEByTN("noInherit"))
@@ -696,7 +687,7 @@ class Manifest(object):
                     self.version = tuple(int(i) for i in version.split("."))
                 self.publicKeyToken = assemblyIdentity.getA("publicKeyToken") or None
             for publisherPolicy in rootElement.getCEByTN("publisherPolicy"):
-                self.applyPublisherPolicy = (publisherPolicy.getA("apply") or 
+                self.applyPublisherPolicy = (publisherPolicy.getA("apply") or
                                              "").lower() == "yes"
             for description in rootElement.getCEByTN("description"):
                 if description.firstChild:
@@ -706,7 +697,7 @@ class Manifest(object):
                     for reqPriv in security.getCEByTN("requestedPrivileges"):
                         for reqExeLev in reqPriv.getCEByTN("requestedExecutionLevel"):
                             self.requestedExecutionLevel = reqExeLev.getA("level")
-                            self.uiAccess = (reqExeLev.getA("uiAccess") or 
+                            self.uiAccess = (reqExeLev.getA("uiAccess") or
                                              "").lower() == "true"
             if rootElement.tagName == "assemblyBinding":
                 dependencies = [rootElement]
@@ -719,11 +710,11 @@ class Manifest(object):
                     if not manifest.name:
                         # invalid, skip
                         continue
-                    manifest.optional = (dependency.getA("optional") or 
+                    manifest.optional = (dependency.getA("optional") or
                                          "").lower() == "yes"
                     self.dependentAssemblies.append(manifest)
                     if self.filename:
-                        # Enable search for private assembly by assigning bogus 
+                        # Enable search for private assembly by assigning bogus
                         # filename (only the directory has to be correct)
                         self.dependentAssemblies[-1].filename = ":".join(
                             (self.filename, manifest.name))
@@ -739,7 +730,7 @@ class Manifest(object):
                 self.add_file(name=file_.getA("name"),
                               hashalg=file_.getA("hashalg"),
                               hash=file_.getA("hash"))
-    
+
     def parse(self, filename_or_file, initialize=True):
         """ Load manifest from file or file object """
         if isinstance(filename_or_file, string_types):
@@ -759,7 +750,7 @@ class Manifest(object):
             self.__init__()
         self.filename = filename
         self.load_dom(domtree, False)
-    
+
     def parse_string(self, xmlstr, initialize=True):
         """ Load manifest from XML string """
         try:
@@ -767,14 +758,14 @@ class Manifest(object):
         except xml.parsers.expat.ExpatError as e:
             raise ManifestXMLParseError(e)
         self.load_dom(domtree, initialize)
-    
+
     def same_id(self, manifest, skip_version_check=False):
         """
         Return a bool indicating if another manifest has the same identitiy.
-        
-        This is done by comparing language, name, processorArchitecture, 
+
+        This is done by comparing language, name, processorArchitecture,
         publicKeyToken, type and version.
-        
+
         """
         if skip_version_check:
             version_check = True
@@ -786,7 +777,7 @@ class Manifest(object):
                 self.publicKeyToken == manifest.publicKeyToken and
                 self.type == manifest.type and
                 version_check)
-    
+
     def todom(self):
         """ Return the manifest as DOM tree """
         doc = Document()
@@ -802,7 +793,7 @@ class Manifest(object):
         if self.manifestType != "dependentAssembly":
             docE.setA("xmlns", "urn:schemas-microsoft-com:asm.v1")
             if self.manifestType != "assemblyBinding":
-                docE.setA("manifestVersion", 
+                docE.setA("manifestVersion",
                           ".".join([str(i) for i in self.manifestVersion]))
         if self.noInheritable:
             docE.aChild(doc.cE("noInheritable"))
@@ -816,10 +807,10 @@ class Manifest(object):
         if self.language:
             aId.setAttribute("language", self.language)
         if self.processorArchitecture:
-            aId.setAttribute("processorArchitecture", 
+            aId.setAttribute("processorArchitecture",
                              self.processorArchitecture)
         if self.version:
-            aId.setAttribute("version", 
+            aId.setAttribute("version",
                              ".".join([str(i) for i in self.version]))
         if self.publicKeyToken:
             aId.setAttribute("publicKeyToken", self.publicKeyToken)
@@ -838,7 +829,7 @@ class Manifest(object):
             descE = doc.cE("description")
             descE.aChild(doc.cT(self.description))
             docE.aChild(descE)
-        if self.requestedExecutionLevel in ("asInvoker", "highestAvailable", 
+        if self.requestedExecutionLevel in ("asInvoker", "highestAvailable",
                                             "requireAdministrator"):
             tE = doc.cE("trustInfo")
             tE.setA("xmlns", "urn:schemas-microsoft-com:asm.v3")
@@ -873,13 +864,13 @@ class Manifest(object):
         if self.bindingRedirects:
             for bindingRedirect in self.bindingRedirects:
                 brE = doc.cE("bindingRedirect")
-                brE.setAttribute("oldVersion", 
-                                 "-".join([".".join([str(i) 
-                                                     for i in 
-                                                     part]) 
-                                           for part in 
+                brE.setAttribute("oldVersion",
+                                 "-".join([".".join([str(i)
+                                                     for i in
+                                                     part])
+                                           for part in
                                            bindingRedirect[0]]))
-                brE.setAttribute("newVersion", 
+                brE.setAttribute("newVersion",
                                  ".".join([str(i) for i in bindingRedirect[1]]))
                 docE.aChild(brE)
         if self.files:
@@ -908,31 +899,31 @@ class Manifest(object):
         docE.aChild(cE)
 
         return doc
-    
+
     def toprettyxml(self, indent="  ", newl=os.linesep, encoding="UTF-8"):
         """ Return the manifest as pretty-printed XML """
         domtree = self.todom()
-        # WARNING: The XML declaration has to follow the order 
-        # version-encoding-standalone (standalone being optional), otherwise 
-        # if it is embedded in an exe the exe will fail to launch! 
+        # WARNING: The XML declaration has to follow the order
+        # version-encoding-standalone (standalone being optional), otherwise
+        # if it is embedded in an exe the exe will fail to launch!
         # ('application configuration incorrect')
         if sys.version_info >= (2,3):
             xmlstr = domtree.toprettyxml(indent, newl, encoding)
         else:
             xmlstr = domtree.toprettyxml(indent, newl)
         xmlstr = xmlstr.decode().strip(os.linesep).replace(
-                '<?xml version="1.0" encoding="%s"?>' % encoding, 
-                '<?xml version="1.0" encoding="%s" standalone="yes"?>' % 
+                '<?xml version="1.0" encoding="%s"?>' % encoding,
+                '<?xml version="1.0" encoding="%s" standalone="yes"?>' %
                 encoding)
         domtree.unlink()
         return xmlstr
-    
+
     def toxml(self, encoding="UTF-8"):
         """ Return the manifest as XML """
         domtree = self.todom()
-        # WARNING: The XML declaration has to follow the order 
-        # version-encoding-standalone (standalone being optional), otherwise 
-        # if it is embedded in an exe the exe will fail to launch! 
+        # WARNING: The XML declaration has to follow the order
+        # version-encoding-standalone (standalone being optional), otherwise
+        # if it is embedded in an exe the exe will fail to launch!
         # ('application configuration incorrect')
         xmlstr = domtree.toxml(encoding).decode().replace(
                 '<?xml version="1.0" encoding="%s"?>' % encoding,
@@ -943,10 +934,11 @@ class Manifest(object):
 
     def update_resources(self, dstpath, names=None, languages=None):
         """ Update or add manifest resource in dll/exe file dstpath """
-        UpdateManifestResourcesFromXML(dstpath, self.toprettyxml(), names, 
-                                       languages)
-    
-    def writeprettyxml(self, filename_or_file=None, indent="  ", newl=os.linesep, 
+        UpdateManifestResourcesFromXML(dstpath,
+                                       self.toprettyxml().encode("UTF-8"),
+                                       names, languages)
+
+    def writeprettyxml(self, filename_or_file=None, indent="  ", newl=os.linesep,
                        encoding="UTF-8"):
         """ Write the manifest as XML to a file or file object """
         if not filename_or_file:
@@ -954,10 +946,10 @@ class Manifest(object):
         if isinstance(filename_or_file, string_types):
             filename_or_file = open(filename_or_file, "wb")
         xmlstr = self.toprettyxml(indent, newl, encoding)
-        filename_or_file.write(xmlstr.encode())
-        filename_or_file.close()
-    
-    def writexml(self, filename_or_file=None, indent="  ", newl=os.linesep, 
+        with filename_or_file:
+            filename_or_file.write(xmlstr.encode())
+
+    def writexml(self, filename_or_file=None, indent="  ", newl=os.linesep,
                  encoding="UTF-8"):
         """ Write the manifest as XML to a file or file object """
         if not filename_or_file:
@@ -965,8 +957,8 @@ class Manifest(object):
         if isinstance(filename_or_file, string_types):
             filename_or_file = open(filename_or_file, "wb")
         xmlstr = self.toxml(encoding)
-        filename_or_file.write(xmlstr.encode())
-        filename_or_file.close()
+        with filename_or_file:
+            filename_or_file.write(xmlstr.encode())
 
 
 def ManifestFromResFile(filename, names=None, languages=None):
@@ -979,7 +971,7 @@ def ManifestFromResFile(filename, names=None, languages=None):
             pth.append(str(key))
             res = res[key]
     if isinstance(res, dict):
-        raise InvalidManifestError("No matching manifest resource found in '%s'" % 
+        raise InvalidManifestError("No matching manifest resource found in '%s'" %
                                    filename)
     manifest = Manifest()
     manifest.filename = ":".join([filename] + pth)
@@ -1013,28 +1005,28 @@ def GetManifestResources(filename, names=None, languages=None):
     return winresource.GetResources(filename, [RT_MANIFEST], names, languages)
 
 
-def UpdateManifestResourcesFromXML(dstpath, xmlstr, names=None, 
+def UpdateManifestResourcesFromXML(dstpath, xmlstr, names=None,
                                    languages=None):
     """ Update or add manifest XML as resource in dstpath """
     logger.info("Updating manifest in %s", dstpath)
     if dstpath.lower().endswith(".exe"):
-        name = 1 
+        name = 1
     else:
         name = 2
-    winresource.UpdateResources(dstpath, xmlstr, RT_MANIFEST, names or [name], 
+    winresource.UpdateResources(dstpath, xmlstr, RT_MANIFEST, names or [name],
                              languages or [0, "*"])
 
 
-def UpdateManifestResourcesFromXMLFile(dstpath, srcpath, names=None, 
+def UpdateManifestResourcesFromXMLFile(dstpath, srcpath, names=None,
                                        languages=None):
     """ Update or add manifest XML from srcpath as resource in dstpath """
     logger.info("Updating manifest from %s in %s", srcpath, dstpath)
     if dstpath.lower().endswith(".exe"):
-        name = 1 
+        name = 1
     else:
         name = 2
-    winresource.UpdateResourcesFromDataFile(dstpath, srcpath, RT_MANIFEST, 
-                                         names or [name], 
+    winresource.UpdateResourcesFromDataFile(dstpath, srcpath, RT_MANIFEST,
+                                         names or [name],
                                          languages or [0, "*"])
 
 
@@ -1109,7 +1101,7 @@ def processor_architecture():
         return 'amd64'
 
 
-if __name__ == "__main__":    
+if __name__ == "__main__":
     dstpath = sys.argv[1]
     srcpath = sys.argv[2]
     UpdateManifestResourcesFromXMLFile(dstpath, srcpath)
