@@ -1,6 +1,6 @@
 /*
  * ****************************************************************************
- * Copyright (c) 2013-2016, PyInstaller Development Team.
+ * Copyright (c) 2013-2018, PyInstaller Development Team.
  * Distributed under the terms of the GNU General Public License with exception
  * for distributing bootloader.
  *
@@ -77,7 +77,12 @@ splitName(char *path, char *filename, const char *item)
     char name[PATH_MAX + 1];
 
     VS("LOADER: Splitting item into path and filename\n");
-    strcpy(name, item);
+    strncpy(name, item, PATH_MAX + 1);
+
+    if (name[PATH_MAX] != '\0') {
+        return -1;
+    }
+
     strcpy(path, strtok(name, ":"));
     strcpy(filename, strtok(NULL, ":"));
 
@@ -136,13 +141,22 @@ _get_archive(ARCHIVE_STATUS *archive_pool[], const char *path)
     archive = (ARCHIVE_STATUS *) malloc(sizeof(ARCHIVE_STATUS));
 
     if (archive == NULL) {
-        FATALERROR("Error allocating memory for status\n");
+        FATAL_PERROR("malloc", "Error allocating memory for status\n");
+        return NULL;
+    }
+
+    strncpy(archive->archivename, path, PATH_MAX);
+    strncpy(archive->homepath, archive_pool[SELF]->homepath, PATH_MAX);
+    strncpy(archive->temppath, archive_pool[SELF]->temppath, PATH_MAX);
+
+    if (archive->archivename[PATH_MAX-1] != '\0'
+        || archive->homepath[PATH_MAX-1] != '\0'
+        || archive->temppath[PATH_MAX-1] != '\0') {
+        FATALERROR("Archive path exceeds PATH_MAX\n");
+        free(archive);
         return NULL;
     }
 
-    strcpy(archive->archivename, path);
-    strcpy(archive->homepath, archive_pool[SELF]->homepath);
-    strcpy(archive->temppath, archive_pool[SELF]->temppath);
     /*
      * Setting this flag prevents creating another temp directory and
      * the directory from the main archive status is used.
@@ -150,7 +164,7 @@ _get_archive(ARCHIVE_STATUS *archive_pool[], const char *path)
     archive->has_temp_directory = archive_pool[SELF]->has_temp_directory;
 
     if (pyi_arch_open(archive)) {
-        FATALERROR("Error openning archive %s\n", path);
+        FATAL_PERROR("malloc", "Error opening archive %s\n", path);
         free(archive);
         return NULL;
     }
@@ -214,7 +228,7 @@ _extract_dependency(ARCHIVE_STATUS *archive_pool[], const char *item)
         VS("LOADER: File %s found, assuming is onedir\n", srcpath);
 
         if (copyDependencyFromDir(archive_status, srcpath, filename) == -1) {
-            FATALERROR("Error coping %s\n", filename);
+            FATALERROR("Error copying %s\n", filename);
             return -1;
         }
         /* TODO implement pyi_path_join to accept variable length of arguments for this case. */
@@ -224,7 +238,7 @@ _extract_dependency(ARCHIVE_STATUS *archive_pool[], const char *item)
         VS("LOADER: File %s found, assuming is onedir\n", srcpath);
 
         if (copyDependencyFromDir(archive_status, srcpath, filename) == -1) {
-            FATALERROR("Error coping %s\n", filename);
+            FATALERROR("Error copying %s\n", filename);
             return -1;
         }
     }
@@ -350,6 +364,7 @@ pyi_launch_run_scripts(ARCHIVE_STATUS *status)
 {
     unsigned char *data;
     char buf[PATH_MAX];
+    size_t namelen;
     TOC * ptoc = status->tocbuff;
     PyObject *__main__;
     PyObject *__file__;
@@ -377,6 +392,12 @@ pyi_launch_run_scripts(ARCHIVE_STATUS *status)
             data = pyi_arch_extract(status, ptoc);
             /* Set the __file__ attribute within the __main__ module,
              *  for full compatibility with normal execution. */
+            namelen = strnlen(ptoc->name, PATH_MAX);
+            if (namelen >= PATH_MAX-strlen(".py")-1) {
+                FATALERROR("Name exceeds PATH_MAX\n");
+                return -1;
+            }
+
             strcpy(buf, ptoc->name);
             strcat(buf, ".py");
             VS("LOADER: Running %s\n", buf);
@@ -482,15 +503,7 @@ done:
 void
 pyi_launch_initialize(ARCHIVE_STATUS * status)
 {
-#if defined(__APPLE__) && defined(WINDOWED)
-    /*
-     * On OS X this ensures that the application is handled as GUI app.
-     * Call TransformProcessType() in the child process.
-     */
-    ProcessSerialNumber psn = { 0, kCurrentProcess };
-    OSStatus returnCode = TransformProcessType(&psn,
-                                               kProcessTransformToForegroundApplication);
-#elif defined(_WIN32)
+#if defined(_WIN32)
     char * manifest;
     manifest = pyi_arch_get_option(status, "pyi-windows-manifest-filename");
 
@@ -499,7 +512,7 @@ pyi_launch_initialize(ARCHIVE_STATUS * status)
         CreateActContext(manifest);
         free(manifest);
     }
-#endif /* if defined(__APPLE__) && defined(WINDOWED) */
+#endif /* if defined(_WIN32) */
 }
 
 /*
