@@ -1,6 +1,6 @@
 /*
  * ****************************************************************************
- * Copyright (c) 2013-2016, PyInstaller Development Team.
+ * Copyright (c) 2013-2018, PyInstaller Development Team.
  * Distributed under the terms of the GNU General Public License with exception
  * for distributing bootloader.
  *
@@ -44,6 +44,13 @@
     #include <sys/wait.h>
     #include <unistd.h>  /* rmdir, unlink, mkdtemp */
 #endif /* ifdef _WIN32 */
+#ifndef SIGCLD
+#define SIGCLD SIGCHLD /* not defined on OS X */
+#endif
+#ifndef sighandler_t
+typedef void (*sighandler_t)(int);
+#endif
+#include <errno.h>
 #include <stddef.h> /* ptrdiff_t */
 #include <stdio.h>  /* FILE */
 #include <stdlib.h>
@@ -60,8 +67,10 @@
  * - On AIX 5.2 function 'mkdtemp' is missing. It is there in version 6.1 but we don't know
  *   the runtime platform at compile time, so we always include our own implementation on AIX.
  */
-#if defined(SUNOS) || defined(AIX)
+#if defined(SUNOS) || defined(AIX) || defined(HPUX)
+    #if !defined(HAVE_MKDTEMP)
     #include "mkdtemp.h"
+    #endif
 #endif
 
 /* PyInstaller headers. */
@@ -89,9 +98,39 @@ static int argc_pyi = 0;
  * on the App icon in the OS X dock.
  */
 #if defined(__APPLE__) && defined(WINDOWED)
-static void process_apple_events_new();
+static void process_apple_events();
 #endif
 
+char *
+pyi_strjoin(const char *first, const char *sep, const char *second){
+    /* join first and second string, using sep as separator.
+     * any of them may be either a null-terminated string or NULL.
+     * sep will be only used if first and second string are not empty.
+     * returns a null-terminated string which the caller is responsible
+     * for freeing. Returns NULL if memory could not be allocated.
+     */
+    int first_len, sep_len, second_len;
+    char *result;
+    first_len = first ? strlen(first) : 0;
+    sep_len = sep ? strlen(sep) : 0;
+    second_len = second ? strlen(second) : 0;
+    result = malloc(first_len + sep_len + second_len + 1);
+    if (!result) {
+        return NULL;
+    }
+    *result = '\0';
+    if (first_len) {
+        strcat(result, first);
+    }
+    if (sep_len && first_len && second_len) {
+        strcat(result, sep);
+    }
+    if (second_len) {
+        strcat(result, second);
+    }
+    return result;
+}
+
 /* Return string copy of environment variable. */
 char *
 pyi_getenv(const char *variable)
@@ -151,10 +190,14 @@ pyi_setenv(const char *variable, const char *value)
 
 #ifdef _WIN32
     wchar_t * wvar, *wval;
+
     wvar = pyi_win32_utils_from_utf8(NULL, variable, 0);
     wval = pyi_win32_utils_from_utf8(NULL, value, 0);
 
-    rc = SetEnvironmentVariableW(wvar, wval);
+    // Not sure why, but SetEnvironmentVariableW() didn't work with _wtempnam()
+    // Replaced it with _wputenv_s()
+    rc = _wputenv_s(wvar, wval);
+
     free(wvar);
     free(wval);
 #else
@@ -178,7 +221,7 @@ pyi_unsetenv(const char *variable)
     #if HAVE_UNSETENV
     rc = unsetenv(variable);
     #else /* HAVE_UNSETENV */
-    rc = setenv(variable, "");
+    rc = setenv(variable, "", true);
     #endif /* HAVE_UNSETENV */
 #endif     /* _WIN32 */
     return rc;
@@ -188,16 +231,28 @@ pyi_unsetenv(const char *variable)
 
 /* TODO rename fuction and revisit */
 int
-pyi_get_temp_path(char *buffer)
+pyi_get_temp_path(char *buffer, char *runtime_tmpdir)
 {
     int i;
     wchar_t *wchar_ret;
     wchar_t prefix[16];
     wchar_t wchar_buffer[PATH_MAX];
+    char *original_tmpdir;
+    char runtime_tmpdir_abspath[PATH_MAX + 1];
+
+    if (runtime_tmpdir != NULL) {
+      /*
+       * Get original TMP environment variable so it can be restored
+       * after this is done.
+       */
+      original_tmpdir = pyi_getenv("TMP");
+      /*
+       * Set TMP to runtime_tmpdir for _wtempnam() later
+       */
+      pyi_path_fullpath(runtime_tmpdir_abspath, PATH_MAX, runtime_tmpdir);
+      pyi_setenv("TMP", runtime_tmpdir_abspath);
+    }
 
-    /*
-     * Get path to Windows temporary directory.
-     */
     GetTempPathW(PATH_MAX, wchar_buffer);
 
     swprintf(prefix, 16, L"_MEI%d", getpid());
@@ -214,10 +269,32 @@ pyi_get_temp_path(char *buffer)
         if (_wmkdir(wchar_ret) == 0) {
             pyi_win32_utils_to_utf8(buffer, wchar_ret, PATH_MAX);
             free(wchar_ret);
+            if (runtime_tmpdir != NULL) {
+              /*
+               * Restore TMP to what it was
+               */
+              if (original_tmpdir != NULL) {
+                pyi_setenv("TMP", original_tmpdir);
+                free(original_tmpdir);
+              } else {
+                pyi_unsetenv("TMP");
+              }
+            }
             return 1;
         }
         free(wchar_ret);
     }
+    if (runtime_tmpdir != NULL) {
+      /*
+       * Restore TMP to what it was
+       */
+      if (original_tmpdir != NULL) {
+        pyi_setenv("TMP", original_tmpdir);
+        free(original_tmpdir);
+      } else {
+        pyi_unsetenv("TMP");
+      }
+    }
     return 0;
 }
 
@@ -244,36 +321,42 @@ pyi_test_temp_path(char *buff)
 
 /* TODO merge this function with windows version. */
 static int
-pyi_get_temp_path(char *buff)
+pyi_get_temp_path(char *buff, char *runtime_tmpdir)
 {
-    /* On OSX the variable TMPDIR is usually defined. */
-    static const char *envname[] = {
-        "TMPDIR", "TEMP", "TMP", 0
-    };
-    static const char *dirname[] = {
-        "/tmp", "/var/tmp", "/usr/tmp", 0
-    };
-    int i;
-    char *p;
+    if (runtime_tmpdir != NULL) {
+      strcpy(buff, runtime_tmpdir);
+      if (pyi_test_temp_path(buff))
+        return 1;
+    } else {
+      /* On OSX the variable TMPDIR is usually defined. */
+      static const char *envname[] = {
+          "TMPDIR", "TEMP", "TMP", 0
+      };
+      static const char *dirname[] = {
+          "/tmp", "/var/tmp", "/usr/tmp", 0
+      };
+      int i;
+      char *p;
 
-    for (i = 0; envname[i]; i++) {
-        p = pyi_getenv(envname[i]);
+      for (i = 0; envname[i]; i++) {
+          p = pyi_getenv(envname[i]);
 
-        if (p) {
-            strcpy(buff, p);
+          if (p) {
+              strcpy(buff, p);
 
-            if (pyi_test_temp_path(buff)) {
-                return 1;
-            }
-        }
-    }
+              if (pyi_test_temp_path(buff)) {
+                  return 1;
+              }
+          }
+      }
 
-    for (i = 0; dirname[i]; i++) {
-        strcpy(buff, dirname[i]);
+      for (i = 0; dirname[i]; i++) {
+          strcpy(buff, dirname[i]);
 
-        if (pyi_test_temp_path(buff)) {
-            return 1;
-        }
+          if (pyi_test_temp_path(buff)) {
+              return 1;
+          }
+      }
     }
     return 0;
 }
@@ -287,8 +370,15 @@ pyi_get_temp_path(char *buff)
 int
 pyi_create_temp_path(ARCHIVE_STATUS *status)
 {
+    char *runtime_tmpdir = NULL;
+
     if (status->has_temp_directory != true) {
-        if (!pyi_get_temp_path(status->temppath)) {
+        runtime_tmpdir = pyi_arch_get_option(status, "pyi-runtime-tmpdir");
+        if(runtime_tmpdir != NULL) {
+          VS("LOADER: Found runtime-tmpdir %s\n", runtime_tmpdir);
+        }
+
+        if (!pyi_get_temp_path(status->temppath, runtime_tmpdir)) {
             FATALERROR("INTERNAL ERROR: cannot create temporary directory!\n");
             return -1;
         }
@@ -386,7 +476,8 @@ pyi_remove_temp_path(const char *dir)
     struct dirent *finfo;
     int dirnmlen;
 
-    strcpy(fnm, dir);
+    /* Leave 1 char for PY_SEP if needed */
+    strncpy(fnm, dir, PATH_MAX);
     dirnmlen = strlen(fnm);
 
     if (fnm[dirnmlen - 1] != PYI_SEP) {
@@ -436,13 +527,25 @@ pyi_open_target(const char *path, const char* name_)
     char fnm[PATH_MAX];
     char name[PATH_MAX];
     char *dir;
+    size_t len;
 
-    strcpy(fnm, path);
-    strcpy(name, name_);
+    strncpy(fnm, path, PATH_MAX);
+    strncpy(name, name_, PATH_MAX);
 
+    /* Check if the path names could be copied */
+    if (fnm[PATH_MAX-1] != '\0' || name[PATH_MAX-1] != '\0') {
+        return NULL;
+    }
+
+    len = strlen(fnm);
     dir = strtok(name, PYI_SEPSTR);
 
     while (dir != NULL) {
+        len += strlen(dir) + strlen(PYI_SEPSTR);
+        /* Check if fnm does not exceed the buffer size */
+        if (len >= PATH_MAX-1) {
+            return NULL;
+        }
         strcat(fnm, PYI_SEPSTR);
         strcat(fnm, dir);
         dir = strtok(NULL, PYI_SEPSTR);
@@ -494,6 +597,12 @@ pyi_copy_file(const char *src, const char *dst, const char *filename)
     int error = 0;
 
     if (in == NULL || out == NULL) {
+        if (in) {
+            fclose(in);
+        }
+        if (out) {
+            fclose(out);
+        }
         return -1;
     }
 
@@ -567,7 +676,8 @@ pyi_utils_set_environment(const ARCHIVE_STATUS *status)
 }
 
 int
-pyi_utils_create_child(const char *thisfile, const int argc, char *const argv[])
+pyi_utils_create_child(const char *thisfile, const ARCHIVE_STATUS* status,
+                       const int argc, char *const argv[])
 {
     SECURITY_ATTRIBUTES sa;
     STARTUPINFOW si;
@@ -618,7 +728,7 @@ pyi_utils_create_child(const char *thisfile, const int argc, char *const argv[])
         GetExitCodeProcess(pi.hProcess, (unsigned long *)&rc);
     }
     else {
-        FATALERROR("Error creating child process!\n");
+        FATAL_WINERROR("CreateProcessW", "Error creating child process!\n");
         rc = -1;
     }
     return rc;
@@ -630,17 +740,35 @@ static int
 set_dynamic_library_path(const char* path)
 {
     int rc = 0;
+    char *env_var, *env_var_orig;
+    char *new_path, *orig_path;
 
     #ifdef AIX
     /* LIBPATH is used to look up dynamic libraries on AIX. */
-    pyi_setenv("LIBPATH", path);
-    VS("LOADER: LIBPATH=%s\n", path);
+    env_var = "LIBPATH";
+    env_var_orig = "LIBPATH_ORIG";
     #else
     /* LD_LIBRARY_PATH is used on other *nix platforms (except Darwin). */
-    rc = pyi_setenv("LD_LIBRARY_PATH", path);
-    VS("LOADER: LD_LIBRARY_PATH=%s\n", path);
+    env_var = "LD_LIBRARY_PATH";
+    env_var_orig = "LD_LIBRARY_PATH_ORIG";
     #endif /* AIX */
 
+    /* keep original value in a new env var so the application can restore it
+     * before forking subprocesses. This is important so that e.g. a forked
+     * (system installed) ssh can find the matching (system installed) ssh
+     * related libraries - not the potentially different versions of same libs
+     * that we have bundled.
+     */
+    orig_path = pyi_getenv(env_var);
+    if (orig_path) {
+        pyi_setenv(env_var_orig, orig_path);
+        VS("LOADER: %s=%s\n", env_var_orig, orig_path);
+    }
+    /* prepend our path to the original path, pyi_strjoin can deal with orig_path being NULL or empty string */
+    new_path = pyi_strjoin(path, ":", orig_path);
+    rc = pyi_setenv(env_var, new_path);
+    VS("LOADER: %s=%s\n", env_var, new_path);
+    free(new_path);
     return rc;
 }
 
@@ -690,6 +818,38 @@ pyi_utils_set_environment(const ARCHIVE_STATUS *status)
     return rc;
 }
 
+/*
+ * If the program is actived by a systemd socket, systemd will set
+ * LISTEN_PID, LISTEN_FDS environment variable for that process.
+ *
+ * LISTEN_PID is set to the pid of the parent process of bootloader,
+ * which is forked by systemd.
+ *
+ * Bootloader will duplicate LISTEN_FDS to child process, but the
+ * LISTEN_PID environment variable remains unchanged.
+ *
+ * Here we change the LISTEN_PID to the child pid in child process.
+ * So the application can detecte it and use the LISTEN_FDS created
+ * by systemd.
+ */
+int
+set_systemd_env()
+{
+    const char * env_var = "LISTEN_PID";
+    if(pyi_getenv(env_var) != NULL) {
+        /* the ULONG_STRING_SIZE is roughly equal to log10(max number)
+         * but can be calculated in compile time.
+         * The idea is from an answer on stackoverflow,
+         * https://stackoverflow.com/questions/8257714/
+         */
+        #define ULONG_STRING_SIZE (sizeof (unsigned long) * CHAR_BIT / 3 + 2)
+        char pid_str[ULONG_STRING_SIZE];
+        snprintf(pid_str, ULONG_STRING_SIZE, "%ld", (unsigned long)getpid());
+        return pyi_setenv(env_var, pid_str);
+    }
+    return 0;
+}
+
 /* Remember child process id. It allows sending a signal to child process.
  * Frozen application always runs in a child process. Parent process is used
  * to setup environment for child process and clean the environment when
@@ -698,21 +858,41 @@ pyi_utils_set_environment(const ARCHIVE_STATUS *status)
 pid_t child_pid = 0;
 
 static void
-_signal_handler(int signal)
+_ignoring_signal_handler(int signum)
 {
-    kill(child_pid, signal);
+    VS("LOADER: Ignoring signal %d\n", signum);
+}
+
+static void
+_signal_handler(int signum)
+{
+    VS("LOADER: Forwarding signal %d to child pid %d\n", signum, child_pid);
+    kill(child_pid, signum);
 }
 
 /* Start frozen application in a subprocess. The frozen application runs
  * in a subprocess.
  */
 int
-pyi_utils_create_child(const char *thisfile, const int argc, char *const argv[])
+pyi_utils_create_child(const char *thisfile, const ARCHIVE_STATUS* status,
+                       const int argc, char *const argv[])
 {
     pid_t pid = 0;
     int rc = 0;
     int i;
 
+    /* cause nonzero return unless this is overwritten
+     * with a successful return code from wait() */
+    int wait_rc = -1;
+
+    /* As indicated in signal(7), signal numbers range from 1-31 (standard)
+     * and 32-64 (Linux real-time). */
+    const size_t num_signals = 65;
+
+    sighandler_t handler;
+    int ignore_signals;
+    int signum;
+
     argv_pyi = (char**)calloc(argc + 1, sizeof(char*));
     argc_pyi = 0;
 
@@ -731,47 +911,85 @@ pyi_utils_create_child(const char *thisfile, const int argc, char *const argv[])
     }
 
     #if defined(__APPLE__) && defined(WINDOWED)
-    process_apple_events_new();
+    process_apple_events();
     #endif
 
     pid = fork();
+    if (pid < 0) {
+        VS("LOADER: failed to fork child process: %s\n", strerror(errno));
+        goto cleanup;
+    }
 
     /* Child code. */
     if (pid == 0) {
         /* Replace process by starting a new application. */
-        execvp(thisfile, argv_pyi);
-    }
-    /* Parent code. */
-    else {
-        child_pid = pid;
-
-        /* Redirect termination signals received by parent to child process. */
-        signal(SIGINT, &_signal_handler);
-        signal(SIGKILL, &_signal_handler);
-        signal(SIGTERM, &_signal_handler);
-    }
-
-    wait(&rc);
-
-    /* Parent code. */
-    if (child_pid != 0) {
-        /* When child process exited, reset signal handlers to default values. */
-        signal(SIGINT, SIG_DFL);
-        signal(SIGKILL, SIG_DFL);
-        signal(SIGTERM, SIG_DFL);
-
-        for (i = 0; i < argc_pyi; i++) {
-            free(argv_pyi[i]);
+        if (set_systemd_env() != 0) {
+            VS("WARNING: Application is started by systemd socket,"
+               "but we can't set proper LISTEN_PID on it.\n");
         }
-        free(argv_pyi);
+        if (execvp(thisfile, argv_pyi) < 0) {
+            VS("Failed to exec: %s\n", strerror(errno));
+            goto cleanup;
+        }
+        /* NOTREACHED */
+    }
+
+    /* From here to end-of-function is parent code (since the child exec'd).
+     * The exception is the `cleanup` block that frees argv_pyi; in the child,
+     * wait_rc is -1, so the child exit code checking is skipped. */
+
+    child_pid = pid;
+    ignore_signals = (pyi_arch_get_option(status, "pyi-bootloader-ignore-signals") != NULL);
+    handler = ignore_signals ? &_ignoring_signal_handler : &_signal_handler;
+
+    /* Redirect all signals received by parent to child process. */
+    if (ignore_signals) {
+        VS("LOADER: Ignoring all signals in parent\n");
+    } else {
+        VS("LOADER: Registering signal handlers\n");
+    }
+    for (signum = 0; signum < num_signals; ++signum) {
+        // don't mess with SIGCHLD/SIGCLD; it affects our ability
+        // to wait() for the child to exit
+        if (signum != SIGCHLD && signum != SIGCLD) {
+            signal(signum, handler);
+        }
+    }
+
+    wait_rc = waitpid(child_pid, &rc, 0);
+    if (wait_rc < 0) {
+        VS("LOADER: failed to wait for child process: %s\n", strerror(errno));
+    }
+
+    /* When child process exited, reset signal handlers to default values. */
+    VS("LOADER: Restoring signal handlers\n");
+    for (signum = 0; signum < num_signals; ++signum) {
+        signal(signum, SIG_DFL);
+    }
+
+  cleanup:
+    VS("LOADER: freeing args\n");
+    for (i = 0; i < argc_pyi; i++) {
+        free(argv_pyi[i]);
+    }
+    free(argv_pyi);
+
+    /* Either wait() failed, or we jumped to `cleanup` and
+     * didn't wait() at all. Either way, exit with error,
+     * because rc does not contain a valid process exit code. */
+    if (wait_rc < 0) {
+        VS("LOADER: exiting early\n");
+        return 1;
     }
 
     if (WIFEXITED(rc)) {
+        VS("LOADER: returning child exit status %d\n", WEXITSTATUS(rc));
         return WEXITSTATUS(rc);
     }
 
     /* Process ended abnormally */
     if (WIFSIGNALED(rc)) {
+        VS("LOADER: re-raising child signal %d\n", WTERMSIG(rc));
         /* Mimick the signal the child received */
         raise(WTERMSIG(rc));
     }
@@ -781,251 +999,128 @@ pyi_utils_create_child(const char *thisfile, const int argc, char *const argv[])
 /*
  * On Mac OS X this converts files from kAEOpenDocuments events into sys.argv.
  */
-    #if defined(__APPLE__) && defined(WINDOWED)
+#if defined(__APPLE__) && defined(WINDOWED)
 
-/* TODO: Why is this commented out? */
-/*
- *  static pascal OSErr handle_open_doc_ae(const AppleEvent *theAppleEvent, AppleEvent *reply, SRefCon handlerRefcon)
- *  {
- *  AEDescList docList;
- *  long index;
- *  long count = 0;
- *  int i;
- *  char *myFileName;
- *  Size actualSize;
- *  DescType returnedType;
- *  AEKeyword keywd;
- *  FSRef theRef;
- *
- *  VS("LOADER: handle_open_doc_ae called.\n");
- *
- *  OSErr err = AEGetParamDesc(theAppleEvent, keyDirectObject, typeAEList, &docList);
- *  if (err != noErr) return err;
- *
- *  err = AECountItems(&docList, &count);
- *  if (err != noErr) return err;
- *  for (index = 1; index <= count; index++)
- *  {
- *    err = AEGetNthPtr(&docList, index, typeFSRef, &keywd, &returnedType, &theRef, sizeof(theRef), &actualSize);
- *
- *    CFURLRef fullURLRef;
- *    fullURLRef = CFURLCreateFromFSRef(NULL, &theRef);
- *    CFStringRef cfString = CFURLCopyFileSystemPath(fullURLRef, kCFURLPOSIXPathStyle);
- *    CFRelease(fullURLRef);
- *    CFMutableStringRef cfMutableString = CFStringCreateMutableCopy(NULL, 0, cfString);
- *    CFRelease(cfString);
- *    CFStringNormalize(cfMutableString, kCFStringNormalizationFormC);
- *    int len = CFStringGetLength(cfMutableString);
- *    const int bufferSize = (len+1)*6;  // in theory up to six bytes per Unicode code point, for UTF-8.
- *    char* buffer = (char*)malloc(bufferSize);
- *    CFStringGetCString(cfMutableString, buffer, bufferSize, kCFStringEncodingUTF8);
- *
- *    argv_pyi = (char**)realloc(argv_pyi,(argc_pyi+2)*sizeof(char*));
- *    argv_pyi[argc_pyi++] = strdup(buffer);
- *    argv_pyi[argc_pyi] = NULL;
- *
- *    free(buffer);
- *  }
- *
- *  err = AEDisposeDesc(&docList);
- *
- *
- *  return (err);
- *  }
- *
- *  static int gQuit = false;
- *
- *  static void apple_main_event_loop()
- *  {
- *  OSErr err;
- *  EventRef eventRef;
- *  EventRecord event;
- *  //UInt32 timeout = 1*60; // number of ticks (1/60th of a second)
- *  VS("LOADER: Entering AppleEvent main loop.\n");
- *
- *  while (!gQuit)
- *  {
- *     // Previous func WaitNextEvent() was deprecated and 32-bit only.
- *     // Func ReceiveNextEvent() is threadsafe and available even for 64-bit OS X.
- *     //gotEvent = WaitNextEvent(highLevelEventMask, &event, timeout, NULL);
- *     err = ReceiveNextEvent(0, NULL, kEventDurationNoWait, true, &eventRef);
- *     if (err == noErr)
- *     {
- *        VS("LOADER: Processing an AppleEvent.\n");
- *        // Convert the event ref to the type AEProcessAppleEvent expects.
- *        ConvertEventRefToEventRecord(eventRef, &event);
- *        AEProcessAppleEvent(&event);
- *        // Func ReceiveNextEvent() requires releasing event manually.
- *        ReleaseEvent(eventRef);
- *     }
- *     gQuit = true;
- *  }
- *  }
- *
- *
- *  static void process_apple_events()
- *  {
- *  OSErr err;
- *
- *  err = AEInstallEventHandler( kCoreEventClass , kAEOpenDocuments , handle_open_doc_ae , 0 , false );
- *  if (err != noErr)
- *   {
- *      VS("LOADER: Error installing AppleEvent handler.\n");
- *   }
- *   else
- *   {
- *      apple_main_event_loop();
- *
- *      err = AERemoveEventHandler(kCoreEventClass, kAEOpenDocuments, handle_open_doc_ae, false);
- *      if (err != noErr)
- *      {
- *         VS("LOADER: Error uninstalling AppleEvent handler.\n");
- *      }
- *   }
- *
- *  }
- */
-/*
- * On Mac OS X converts files from kAEOpenDocuments events into sys.argv.
- * New implementation of handling Apple events - works even on 64bit Mac OS X.
- */
-static pascal OSStatus
-do_filenames_conversion_to_argv(EventHandlerCallRef next_handler,
-                                EventRef the_event,
-                                void* user_data)
+static int gQuit = false;
+
+static pascal OSErr handle_open_doc_ae(const AppleEvent *theAppleEvent, AppleEvent *reply, SRefCon handlerRefcon)
 {
+   AEDescList docList;
+   long index;
+   long count = 0;
+   int i;
+   char *myFileName;
+   Size actualSize;
+   DescType returnedType;
+   AEKeyword keywd;
+   FSRef theRef;
 
-/* /////////////// OLD implementation */
-/*
- *  AEDescList docList;
- *  long index;
- *  long count = 0;
- *  int i;
- *  char *myFileName;
- *  Size actualSize;
- *  DescType returnedType;
- *  AEKeyword keywd;
- *  FSRef theRef;
- *
- *  VS("LOADER: handle_open_doc_ae called.\n");
- *
- *  OSErr err = AEGetParamDesc(theAppleEvent, keyDirectObject, typeAEList, &docList);
- *  if (err != noErr) return err;
- *
- *  err = AECountItems(&docList, &count);
- *  if (err != noErr) return err;
- *  for (index = 1; index <= count; index++)
- *  {
- *    err = AEGetNthPtr(&docList, index, typeFSRef, &keywd, &returnedType, &theRef, sizeof(theRef), &actualSize);
- *
- *    CFURLRef fullURLRef;
- *    fullURLRef = CFURLCreateFromFSRef(NULL, &theRef);
- *    CFStringRef cfString = CFURLCopyFileSystemPath(fullURLRef, kCFURLPOSIXPathStyle);
- *    CFRelease(fullURLRef);
- *    CFMutableStringRef cfMutableString = CFStringCreateMutableCopy(NULL, 0, cfString);
- *    CFRelease(cfString);
- *    CFStringNormalize(cfMutableString, kCFStringNormalizationFormC);
- *    int len = CFStringGetLength(cfMutableString);
- *    const int bufferSize = (len+1)*6;  // in theory up to six bytes per Unicode code point, for UTF-8.
- *    char* buffer = (char*)malloc(bufferSize);
- *    CFStringGetCString(cfMutableString, buffer, bufferSize, kCFStringEncodingUTF8);
- *
- *    argv_pyi = (char**)realloc(argv_pyi,(argc_pyi+2)*sizeof(char*));
- *    argv_pyi[argc_pyi++] = strdup(buffer);
- *    argv_pyi[argc_pyi] = NULL;
- *
- *    free(buffer);
- *  }
- *
- *  err = AEDisposeDesc(&docList);
- */
-/* //////////////////////////////////////////// */
+   VS("LOADER [ARGV_EMU]: OpenDocument handler called.\n");
 
-    /* Handle the kAEOpenDocuments event. */
-    VS("LOADER: AppleEvent - do_filenames_coversion_to_argv called.\n");
-    /* Obtain filenames from event. */
-    /* Every event type contains different additional information. */
-    /* TODO */
-    /* GetEventParameter(the_event, ) */
-    /* TODO */
-    /* Report success. */
-    return noErr;
+   OSErr err = AEGetParamDesc(theAppleEvent, keyDirectObject, typeAEList, &docList);
+   if (err != noErr) return err;
+
+   err = AECountItems(&docList, &count);
+   if (err != noErr) return err;
+
+   for (index = 1; index <= count; index++)
+   {
+     err = AEGetNthPtr(&docList, index, typeFSRef, &keywd, &returnedType, &theRef, sizeof(theRef), &actualSize);
+
+     CFURLRef fullURLRef;
+     fullURLRef = CFURLCreateFromFSRef(NULL, &theRef);
+     CFStringRef cfString = CFURLCopyFileSystemPath(fullURLRef, kCFURLPOSIXPathStyle);
+     CFRelease(fullURLRef);
+     CFMutableStringRef cfMutableString = CFStringCreateMutableCopy(NULL, 0, cfString);
+     CFRelease(cfString);
+     CFStringNormalize(cfMutableString, kCFStringNormalizationFormC);
+     int len = CFStringGetLength(cfMutableString);
+     const int bufferSize = (len+1)*6;  // in theory up to six bytes per Unicode code point, for UTF-8.
+     char* buffer = (char*)malloc(bufferSize);
+     CFStringGetCString(cfMutableString, buffer, bufferSize, kCFStringEncodingUTF8);
+
+     argv_pyi = (char**)realloc(argv_pyi,(argc_pyi+2)*sizeof(char*));
+     argv_pyi[argc_pyi++] = strdup(buffer);
+     argv_pyi[argc_pyi] = NULL;
+
+     VS("LOADER [ARGV_EMU]: argv entry appended.");
+
+     free(buffer);
+   }
+
+  err = AEDisposeDesc(&docList);
+
+
+  return (err);
 }
 
-/* TODO Unfinished, issue #1309. */
-static void
-process_apple_events_new()
+
+static void process_apple_events()
 {
     OSStatus handler_install_status;
     OSStatus handler_remove_status;
-    OSStatus event_received;
+    OSStatus rcv_status;
+    OSStatus pcs_status;
     EventTypeSpec event_types[1];  /*  List of event types to handle. */
-    EventHandlerUPP handler_open_doc;
+    AEEventHandlerUPP handler_open_doc;
     EventHandlerRef handler_ref; /* Reference for later removing the event handler. */
     EventRef event_ref;          /* Event that caused ReceiveNextEvent to return. */
     OSType ev_class;
     UInt32 ev_kind;
-    EventTimeout timeout = 3.0;  /* number of ticks (1/60th of a second) */
+    EventTimeout timeout = 1.0;  /* number of seconds */
 
-    VS("LOADER: AppleEvent - processing...\n");
-    /* Event types we are interested in. */
-    event_types[0].eventClass = kCoreEventClass;
-    event_types[0].eventKind = kAEOpenDocuments;
-    /* Carbon Event Manager requires convert the function pointer to type EventHandlerUPP. */
+    VS("LOADER [ARGV_EMU]: AppleEvent - processing...\n");
+
+    event_types[0].eventClass = kEventClassAppleEvent;
+    event_types[0].eventKind = kEventAppleEvent;
+
+    /* Carbon Event Manager requires us to convert the function pointer to type EventHandlerUPP. */
     /* https://developer.apple.com/legacy/library/documentation/Carbon/Conceptual/Carbon_Event_Manager/Tasks/CarbonEventsTasks.html */
-    handler_open_doc = NewEventHandlerUPP(do_filenames_conversion_to_argv);
-    /* Install the event handler. */
-    /* InstallApplicationEventHandler is a macro for 'InstallEventHandler' where the event target */
-    /* is Application. */
-    /* We do not have any windows so having Application as event target should be fine. */
-    handler_install_status = InstallApplicationEventHandler(handler_open_doc, 1,
-                                                            event_types, NULL,
-                                                            &handler_ref);
+    handler_open_doc = NewAEEventHandlerUPP(handle_open_doc_ae);
+
+    handler_install_status = AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments, handler_open_doc, 0, false);
 
     if (handler_install_status == noErr) {
-        VS("LOADER: AppleEvent - installed handler.\n");
 
-        /* TODO */
-        /* ReceiveNextEvent(); */
-        /* RunCurrentEventLoop(timeout); */
-        /* RunApplicationEventLoop(); */
+        VS("LOADER [ARGV_EMU]: AppleEvent - installed handler.\n");
 
-        /* Previous func WaitNextEvent() was deprecated and 32-bit only. */
-        /* Func ReceiveNextEvent() is threadsafe and available even for 64-bit OS X. */
-        /* gotEvent = WaitNextEvent(highLevelEventMask, &event, timeout, NULL); */
-        /* event_received = ReceiveNextEvent(1, event_types, kEventDurationForever, true, &event_ref); */
-        /* event_received = ReceiveNextEvent(0, NULL, kEventDurationNoWait, true, &event_ref); */
-        /* TODO */
-        /* handler_remove_status = RemoveEventHandler(handler_ref); */
-        /*
-         *  if (event_received == noErr)
-         *  {
-         *   VS("LOADER: AppleEvent - received event.\n");
-         *   ev_class = GetEventClass(event_ref);
-         *   ev_kind = GetEventKind(event_ref);
-         *   VS("LOADER: AppleEvent - event type: %u\n", ev_kind);
-         *   //if (GetEventKind(event_ref) == kAEOpenDocuments)
-         *   if (ev_class == kCoreEventClass)
-         *   {
-         *       VS("LOADER: AppleEvent - right class.\n");
-         *   }
-         *
-         *   ReleaseEvent(event_ref);
-         *  }
-         *  else
-         *  {
-         *   VS("LOADER: AppleEvent - ERROR receiving events code: %d\n", event_received);
-         *  }*/
+        while(!gQuit) {
+           VS("LOADER [ARGV_EMU]: AppleEvent - calling ReceiveNextEvent\n");
+           rcv_status = ReceiveNextEvent(1, event_types, timeout, true, &event_ref);
+
+           if (rcv_status == eventLoopTimedOutErr) {
+              VS("LOADER [ARGV_EMU]: ReceiveNextEvent timed out\n");
+              break;
+           }
+           else if (rcv_status != 0) {
+              VS("LOADER [ARGV_EMU]: ReceiveNextEvent fetching events failed");
+              break;
+           }
+           else
+           {
+              VS("LOADER [ARGV_EMU]: ReceiveNextEvent got an event");
+
+              pcs_status = AEProcessEvent(event_ref);
+              if (pcs_status != 0) {
+                 VS("LOADER [ARGV_EMU]: processing events failed");
+                 break;
+              }
+           }
+        }
+
+        VS("LOADER [ARGV_EMU]: Out of the event loop.");
+
+        handler_remove_status = RemoveEventHandler(handler_ref);
 
     }
     else {
-        VS("LOADER: AppleEvent - ERROR installing handler.\n");
+        VS("LOADER [ARGV_EMU]: AppleEvent - ERROR installing handler.\n");
     }
 
     /* Remove handler_ref reference when we are done with EventHandlerUPP. */
     /* Carbon Event Manager does not do this automatically. */
     DisposeEventHandlerUPP(handler_open_doc)
 }
-    #endif /* if defined(__APPLE__) && defined(WINDOWED) */
+#endif /* if defined(__APPLE__) && defined(WINDOWED) */
 
 #endif  /* WIN32 */
