@@ -1,6 +1,6 @@
 /*
  * ****************************************************************************
- * Copyright (c) 2013-2016, PyInstaller Development Team.
+ * Copyright (c) 2013-2018, PyInstaller Development Team.
  * Distributed under the terms of the GNU General Public License with exception
  * for distributing bootloader.
  *
@@ -46,26 +46,52 @@ static ULONG_PTR actToken;
     #define STATUS_SXS_EARLY_DEACTIVATION 0xC015000F
 #endif
 
-char *
-GetWinErrorString()
-{
-    DWORD error_code = GetLastError();
-    char * errorString = NULL;
+#define ERROR_STRING_MAX 4096
+static char errorString[ERROR_STRING_MAX];
 
-    FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, /* dwFlags */
-                   NULL,                                                        /* lpSource */
-                   error_code,                                                  /* dwMessageID, */
-                   0,                                                           /* dwLanguageID, */
-                   (char *)(&errorString),                                      /* lpBuffer; see FORMAT_MESSAGE_ALLOCATE_BUFFER */
-                   0,                                                           /* nSize */
-                   NULL                                                         /* Arguments */
-                   );
+/* GetWinErrorString
+ *
+ * Return a pointer to a null-terminated string containing a textual description of the
+ * given error code. If the error code is zero, the result of GetLastError() is used.
+ * The text is localized and ANSI-encoded. The caller is not responsible for freeing
+ * this pointer.
+ *
+ * Returns a pointer to statically-allocated storage. Not thread safe.
+ */
 
-    if (NULL == errorString) {
-        return "FormatMessage failed.";
+char * GetWinErrorString(DWORD error_code) {
+    wchar_t local_buffer[ERROR_STRING_MAX];
+    DWORD result;
+
+    if (error_code == 0) {
+        error_code = GetLastError();
+    }
+    /* Note: Giving 0 to dwLanguageID means MAKELANGID(LANG_NEUTRAL,
+     * SUBLANG_NEUTRAL), but we should use SUBLANG_DEFAULT instead of
+     * SUBLANG_NEUTRAL. Please see the note written in
+     * "Language Identifier Constants and Strings" on MSDN.
+     * https://docs.microsoft.com/en-us/windows/desktop/intl/language-identifier-constants-and-strings
+     */
+    result = FormatMessageW(
+        FORMAT_MESSAGE_FROM_SYSTEM, // dwFlags
+        NULL,                       // lpSource
+        error_code,                 // dwMessageID
+        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // dwLanguageID
+        local_buffer,               // lpBuffer
+        ERROR_STRING_MAX,           // nSize
+        NULL                        // Arguments
+        );
+
+    if (!result) {
+        FATAL_WINERROR("FormatMessageW", "No error messages generated.\n");
+        return "PyInstaller: FormatMessageW failed.";
+    }
+    if (!pyi_win32_utils_to_utf8(errorString,
+                                 local_buffer,
+                                 ERROR_STRING_MAX)) {
+        return "PyInstaller: pyi_win32_utils_to_utf8 failed.";
     }
     return errorString;
-
 }
 
 int
@@ -111,8 +137,7 @@ CreateActContext(const char *manifestpath)
     }
 
     hCtx = INVALID_HANDLE_VALUE;
-    VS("LOADER: Error activating the context: ActivateActCtx: \n%s\n",
-       GetWinErrorString());
+    VS("LOADER: Error activating the context: ActivateActCtx: \n%s\n", GetWinErrorString(0));
     return 0;
 }
 
@@ -146,10 +171,7 @@ pyi_win32_wcs_to_mbs(const wchar_t *wstr)
                               );
 
     if (0 == len) {
-        FATALERROR("Failed to get ANSI buffer size"
-                   "(WideCharToMultiByte: %s)\n",
-                   GetWinErrorString()
-                   );
+        FATAL_WINERROR("WideCharToMultiByte", "Failed to get ANSI buffer size.\n");
         return NULL;
     }
 
@@ -166,10 +188,7 @@ pyi_win32_wcs_to_mbs(const wchar_t *wstr)
                               );
 
     if (0 == ret) {
-        FATALERROR("Failed to encode filename as ANSI"
-                   "(WideCharToMultiByte: %s)\n",
-                   GetWinErrorString()
-                   );
+        FATAL_WINERROR("WideCharToMultiByte", "Failed to encode filename as ANSI.\n");
         return NULL;
     }
     return str;
@@ -207,7 +226,7 @@ pyi_win32_wcs_to_mbs_sfn(const wchar_t *wstr)
 
     if (!str) {
         VS("Failed to get short path name for filename. GetShortPathNameW: \n%s\n",
-           GetWinErrorString()
+           GetWinErrorString(0)
            );
         str = pyi_win32_wcs_to_mbs(wstr);
     }
@@ -362,9 +381,7 @@ pyi_win32_utils_to_utf8(char *str, const wchar_t *wstr, size_t len)
                                   );
 
         if (0 == len) {
-            FATALERROR("Failed to get UTF-8 buffer size (WideCharToMultiByte: %s)\n",
-                       GetWinErrorString()
-                       );
+            FATAL_WINERROR("WideCharToMultiByte", "Failed to get UTF-8 buffer size.\n");
             return NULL;
         }
 
@@ -385,9 +402,8 @@ pyi_win32_utils_to_utf8(char *str, const wchar_t *wstr, size_t len)
                               );
 
     if (len == 0) {
-        FATALERROR("Failed to encode wchar_t as UTF-8 (WideCharToMultiByte: %s)\n",
-                   GetWinErrorString()
-                   );
+        FATAL_WINERROR("WideCharToMultiByte",
+                       "Failed to encode wchar_t as UTF-8.\n");
         return NULL;
     }
     return output;
@@ -425,9 +441,7 @@ pyi_win32_utils_from_utf8(wchar_t *wstr, const char *str, size_t wlen)
                                    );
 
         if (0 == wlen) {
-            FATALERROR("Failed to get UTF-8 buffer size (WideCharToMultiByte: %s)\n",
-                       GetWinErrorString()
-                       );
+            FATAL_WINERROR("MultiByteToWideChar", "Failed to get wchar_t buffer size.\n");
             return NULL;
         }
 
@@ -446,9 +460,7 @@ pyi_win32_utils_from_utf8(wchar_t *wstr, const char *str, size_t wlen)
                                );
 
     if (wlen == 0) {
-        FATALERROR("Failed to encode wchar_t as UTF-8 (WideCharToMultiByte: %s)\n",
-                   GetWinErrorString()
-                   );
+        FATAL_WINERROR("MultiByteToWideChar", "Failed to decode wchar_t from UTF-8\n");
         return NULL;
     }
     return output;
