@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2013-2016, PyInstaller Development Team.
+# Copyright (c) 2013-2018, PyInstaller Development Team.
 #
 # Distributed under the terms of the GNU General Public License with exception
 # for distributing bootloader.
@@ -18,7 +18,7 @@ import os.path
 
 # Third-party imports
 # -------------------
-import sphinx
+import sphinx.cmd.build
 
 # Local imports
 # -------------
@@ -29,9 +29,7 @@ sphinx_path = os.path.join(get_data_dir(), 'sphinx')
 # Invoke Sphinx. See
 # http://sphinx-doc.org/invocation.html#invocation-of-sphinx-build for more
 # details of the options below.
-ret = sphinx.main([
-   # First param is name of program (anything is fine).
-   '',
+ret = sphinx.cmd.build.main([
    # Rebuild all files.
    '-a', '-E',
    # Produce html output.
